# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import NlpFit


def test_NlpFit_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        clobber=dict(
            argstr="-clobber",
            usedefault=True,
        ),
        config_file=dict(
            argstr="-config_file %s",
            extensions=None,
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        init_xfm=dict(
            argstr="-init_xfm %s",
            extensions=None,
            mandatory=True,
        ),
        input_grid_files=dict(),
        output_xfm=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        source=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        source_mask=dict(
            argstr="-source_mask %s",
            extensions=None,
            mandatory=True,
        ),
        target=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        verbose=dict(
            argstr="-verbose",
        ),
    )
    inputs = NlpFit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_NlpFit_outputs():
    output_map = dict(
        output_grid=dict(
            extensions=None,
        ),
        output_xfm=dict(
            extensions=None,
        ),
    )
    outputs = NlpFit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import Smooth


def test_Smooth_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="--i %s",
            extensions=None,
            mandatory=True,
        ),
        num_iters=dict(
            argstr="--niters %d",
            mandatory=True,
            xor=["surface_fwhm"],
        ),
        proj_frac=dict(
            argstr="--projfrac %s",
            xor=["proj_frac_avg"],
        ),
        proj_frac_avg=dict(
            argstr="--projfrac-avg %.2f %.2f %.2f",
            xor=["proj_frac"],
        ),
        reg_file=dict(
            argstr="--reg %s",
            extensions=None,
            mandatory=True,
        ),
        smoothed_file=dict(
            argstr="--o %s",
            extensions=None,
            genfile=True,
        ),
        subjects_dir=dict(),
        surface_fwhm=dict(
            argstr="--fwhm %f",
            mandatory=True,
            requires=["reg_file"],
            xor=["num_iters"],
        ),
        vol_fwhm=dict(
            argstr="--vol-fwhm %f",
        ),
    )
    inputs = Smooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Smooth_outputs():
    output_map = dict(
        smoothed_file=dict(
            extensions=None,
        ),
    )
    outputs = Smooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..misc import MergeROIs


def test_MergeROIs_inputs():
    input_map = dict(
        in_files=dict(),
        in_index=dict(),
        in_reference=dict(
            extensions=None,
        ),
    )
    inputs = MergeROIs.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MergeROIs_outputs():
    output_map = dict(
        merged_file=dict(
            extensions=None,
        ),
    )
    outputs = MergeROIs.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..brainsuite import SVReg


def test_SVReg_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        atlasFilePrefix=dict(
            argstr="'%s'",
            position=1,
        ),
        curveMatchingInstructions=dict(
            argstr="'-cur %s'",
        ),
        dataSinkDelay=dict(
            argstr="%s",
        ),
        displayModuleName=dict(
            argstr="'-m'",
        ),
        displayTimestamps=dict(
            argstr="'-t'",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        iterations=dict(
            argstr="'-H %d'",
        ),
        keepIntermediates=dict(
            argstr="'-k'",
        ),
        pialSurfaceMaskDilation=dict(
            argstr="'-D %d'",
        ),
        refineOutputs=dict(
            argstr="'-r'",
        ),
        shortMessages=dict(
            argstr="'-gui'",
        ),
        skipToIntensityReg=dict(
            argstr="'-p'",
        ),
        skipToVolumeReg=dict(
            argstr="'-s'",
        ),
        skipVolumetricProcessing=dict(
            argstr="'-S'",
        ),
        subjectFilePrefix=dict(
            argstr="'%s'",
            mandatory=True,
            position=0,
        ),
        useCerebrumMask=dict(
            argstr="'-C'",
        ),
        useManualMaskFile=dict(
            argstr="'-cbm'",
        ),
        useMultiThreading=dict(
            argstr="'-P'",
        ),
        useSingleThreading=dict(
            argstr="'-U'",
        ),
        verbosity0=dict(
            argstr="'-v0'",
            xor=("verbosity0", "verbosity1", "verbosity2"),
        ),
        verbosity1=dict(
            argstr="'-v1'",
            xor=("verbosity0", "verbosity1", "verbosity2"),
        ),
        verbosity2=dict(
            argstr="'v2'",
            xor=("verbosity0", "verbosity1", "verbosity2"),
        ),
    )
    inputs = SVReg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
import configparser

import os
import sys
import subprocess

COMMIT_INFO_FNAME = "COMMIT_INFO.txt"


def pkg_commit_hash(pkg_path):
    """Get short form of commit hash given directory `pkg_path`

    There should be a file called 'COMMIT_INFO.txt' in `pkg_path`.  This is a
    file in INI file format, with at least one section: ``commit hash`` and two
    variables ``archive_subst_hash`` and ``install_hash``.  The first has a
    substitution pattern in it which may have been filled by the execution of
    ``git archive`` if this is an archive generated that way.  The second is
    filled in by the installation, if the installation is from a git archive.

    We get the commit hash from (in order of preference):

    * A substituted value in ``archive_subst_hash``
    * A written commit hash value in ``install_hash`
    * git's output, if we are in a git repository

    If all these fail, we return a not-found placeholder tuple

    Parameters
    ----------
    pkg_path : str
       directory containing package

    Returns
    -------
    hash_from : str
       Where we got the hash from - description
    hash_str : str
       short form of hash
    """
    # Try and get commit from written commit text file
    pth = os.path.join(pkg_path, COMMIT_INFO_FNAME)
    if not os.path.isfile(pth):
        raise IOError("Missing commit info file %s" % pth)
    cfg_parser = configparser.RawConfigParser()
    with open(pth, encoding="utf-8") as fp:
        cfg_parser.read_file(fp)
    archive_subst = cfg_parser.get("commit hash", "archive_subst_hash")
    if not archive_subst.startswith("$Format"):  # it has been substituted
        return "archive substitution", archive_subst
    install_subst = cfg_parser.get("commit hash", "install_hash")
    if install_subst != "":
        return "installation", install_subst
    # maybe we are in a repository
    proc = subprocess.Popen(
        "git rev-parse --short HEAD",
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=pkg_path,
        shell=True,
    )
    repo_commit, _ = proc.communicate()
    if repo_commit:
        return "repository", repo_commit.decode().strip()
    return "(none found)", "<not found>"


def get_pkg_info(pkg_path):
    """Return dict describing the context of this package

    Parameters
    ----------
    pkg_path : str
       path containing __init__.py for package

    Returns
    -------
    context : dict
       with named parameters of interest
    """
    src, hsh = pkg_commit_hash(pkg_path)
    from .info import VERSION
    import networkx
    import nibabel
    import numpy
    import scipy
    import traits

    return dict(
        pkg_path=pkg_path,
        commit_source=src,
        commit_hash=hsh,
        nipype_version=VERSION,
        sys_version=sys.version,
        sys_executable=sys.executable,
        sys_platform=sys.platform,
        numpy_version=numpy.__version__,
        scipy_version=scipy.__version__,
        networkx_version=networkx.__version__,
        nibabel_version=nibabel.__version__,
        traits_version=traits.__version__,
    )



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import SurfaceTransform


def test_SurfaceTransform_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hemi=dict(
            argstr="--hemi %s",
            mandatory=True,
        ),
        out_file=dict(
            argstr="--tval %s",
            extensions=None,
            genfile=True,
        ),
        reshape=dict(
            argstr="--reshape",
        ),
        reshape_factor=dict(
            argstr="--reshape-factor",
        ),
        source_annot_file=dict(
            argstr="--sval-annot %s",
            extensions=None,
            mandatory=True,
            xor=["source_file"],
        ),
        source_file=dict(
            argstr="--sval %s",
            extensions=None,
            mandatory=True,
            xor=["source_annot_file"],
        ),
        source_subject=dict(
            argstr="--srcsubject %s",
            mandatory=True,
        ),
        source_type=dict(
            argstr="--sfmt %s",
            requires=["source_file"],
        ),
        subjects_dir=dict(),
        target_ico_order=dict(
            argstr="--trgicoorder %d",
        ),
        target_subject=dict(
            argstr="--trgsubject %s",
            mandatory=True,
        ),
        target_type=dict(
            argstr="--tfmt %s",
        ),
    )
    inputs = SurfaceTransform.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SurfaceTransform_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = SurfaceTransform.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import Warp


def test_Warp_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        deoblique=dict(
            argstr="-deoblique",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        gridset=dict(
            argstr="-gridset %s",
            extensions=None,
        ),
        in_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        interp=dict(
            argstr="-%s",
        ),
        matparent=dict(
            argstr="-matparent %s",
            extensions=None,
        ),
        mni2tta=dict(
            argstr="-mni2tta",
        ),
        newgrid=dict(
            argstr="-newgrid %f",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        oblique_parent=dict(
            argstr="-oblique_parent %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            keep_extension=True,
            name_source="in_file",
            name_template="%s_warp",
        ),
        outputtype=dict(),
        save_warp=dict(
            requires=["verbose"],
        ),
        tta2mni=dict(
            argstr="-tta2mni",
        ),
        verbose=dict(
            argstr="-verb",
        ),
        zpad=dict(
            argstr="-zpad %d",
        ),
    )
    inputs = Warp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Warp_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
        warp_file=dict(
            extensions=None,
        ),
    )
    outputs = Warp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..brainsuite import Dfs


def test_Dfs_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        curvatureWeighting=dict(
            argstr="-w %f",
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputShadingVolume=dict(
            argstr="-c %s",
            extensions=None,
        ),
        inputVolumeFile=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
        ),
        noNormalsFlag=dict(
            argstr="--nonormals",
        ),
        nonZeroTessellation=dict(
            argstr="-nz",
            xor=("nonZeroTessellation", "specialTessellation"),
        ),
        outputSurfaceFile=dict(
            argstr="-o %s",
            extensions=None,
            genfile=True,
        ),
        postSmoothFlag=dict(
            argstr="--postsmooth",
        ),
        scalingPercentile=dict(
            argstr="-f %f",
        ),
        smoothingConstant=dict(
            argstr="-a %f",
            usedefault=True,
        ),
        smoothingIterations=dict(
            argstr="-n %d",
            usedefault=True,
        ),
        specialTessellation=dict(
            argstr="%s",
            position=-1,
            requires=["tessellationThreshold"],
            xor=("nonZeroTessellation", "specialTessellation"),
        ),
        tessellationThreshold=dict(
            argstr="%f",
        ),
        timer=dict(
            argstr="--timer",
        ),
        verbosity=dict(
            argstr="-v %d",
        ),
        zeroPadFlag=dict(
            argstr="-z",
        ),
    )
    inputs = Dfs.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Dfs_outputs():
    output_map = dict(
        outputSurfaceFile=dict(
            extensions=None,
        ),
    )
    outputs = Dfs.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import AffineInitializer


def test_AffineInitializer_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        dimension=dict(
            argstr="%s",
            position=0,
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_image=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        local_search=dict(
            argstr="%d",
            position=7,
            usedefault=True,
        ),
        moving_image=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=2,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            position=3,
            usedefault=True,
        ),
        principal_axes=dict(
            argstr="%d",
            position=6,
            usedefault=True,
        ),
        radian_fraction=dict(
            argstr="%f",
            position=5,
            usedefault=True,
        ),
        search_factor=dict(
            argstr="%f",
            position=4,
            usedefault=True,
        ),
    )
    inputs = AffineInitializer.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_AffineInitializer_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = AffineInitializer.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Local serial workflow execution
"""

import os
from .base import PluginBase, logger, report_crash, report_nodes_not_run, str2bool
from ..engine.utils import topological_sort


class LinearPlugin(PluginBase):
    """Execute workflow in series"""

    def run(self, graph, config, updatehash=False):
        """Executes a pre-defined pipeline in a serial order.

        Parameters
        ----------

        graph : networkx digraph
            defines order of execution
        """
        import networkx as nx

        try:
            dfs_preorder = nx.dfs_preorder
        except AttributeError:
            dfs_preorder = nx.dfs_preorder_nodes

        if not isinstance(graph, nx.DiGraph):
            raise ValueError("Input must be a networkx digraph object")
        logger.info("Running serially.")
        old_wd = os.getcwd()
        notrun = []
        donotrun = []
        stop_on_first_crash = str2bool(config["execution"]["stop_on_first_crash"])
        errors = []
        nodes, _ = topological_sort(graph)
        for node in nodes:
            endstatus = "end"
            try:
                if node in donotrun:
                    continue
                if self._status_callback:
                    self._status_callback(node, "start")
                node.run(updatehash=updatehash)
            except Exception as exc:
                endstatus = "exception"
                # bare except, but i really don't know where a
                # node might fail
                crashfile = report_crash(node)
                # remove dependencies from queue
                subnodes = [s for s in dfs_preorder(graph, node)]
                notrun.append(
                    {"node": node, "dependents": subnodes, "crashfile": crashfile}
                )
                donotrun.extend(subnodes)
                # Delay raising the crash until we cleaned the house
                errors.append(exc)

                if stop_on_first_crash:
                    break
            finally:
                if self._status_callback:
                    self._status_callback(node, endstatus)

        os.chdir(old_wd)  # Return wherever we were before
        report_nodes_not_run(notrun)
        if errors:
            # If one or more nodes failed, re-rise first of them
            error, cause = errors[0], None
            if isinstance(error, str):
                error = RuntimeError(error)

            if len(errors) > 1:
                error, cause = (
                    RuntimeError(f"{len(errors)} raised. Re-raising first."),
                    error,
                )

            raise error from cause



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ..base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)


class DWISimpleCompareInputSpec(CommandLineInputSpec):
    inputVolume1 = File(
        desc="First input volume (.nhdr or .nrrd)",
        exists=True,
        argstr="--inputVolume1 %s",
    )
    inputVolume2 = File(
        desc="Second input volume (.nhdr or .nrrd)",
        exists=True,
        argstr="--inputVolume2 %s",
    )
    checkDWIData = traits.Bool(
        desc="check for existence of DWI data, and if present, compare it",
        argstr="--checkDWIData ",
    )


class DWISimpleCompareOutputSpec(TraitedSpec):
    pass


class DWISimpleCompare(SEMLikeCommandLine):
    """title: Nrrd DWI comparison

    category: Converters

    description: Compares two nrrd format DWI images and verifies that gradient magnitudes, gradient directions, measurement frame, and max B0 value are identicle.  Used for testing DWIConvert.

    version: 0.1.0.$Revision: 916 $(alpha)

    documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DWIConvert

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Mark Scully (UIowa)

    acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.  Additional support for DTI data produced on Philips scanners was contributed by Vincent Magnotta and Hans Johnson at the University of Iowa.
    """

    input_spec = DWISimpleCompareInputSpec
    output_spec = DWISimpleCompareOutputSpec
    _cmd = " DWISimpleCompare "
    _outputs_filenames = {}
    _redirect_x = False


class DWICompareInputSpec(CommandLineInputSpec):
    inputVolume1 = File(
        desc="First input volume (.nhdr or .nrrd)",
        exists=True,
        argstr="--inputVolume1 %s",
    )
    inputVolume2 = File(
        desc="Second input volume (.nhdr or .nrrd)",
        exists=True,
        argstr="--inputVolume2 %s",
    )


class DWICompareOutputSpec(TraitedSpec):
    pass


class DWICompare(SEMLikeCommandLine):
    """title: Nrrd DWI comparison

    category: Converters

    description: Compares two nrrd format DWI images and verifies that gradient magnitudes, gradient directions, measurement frame, and max B0 value are identicle.  Used for testing DWIConvert.

    version: 0.1.0.$Revision: 916 $(alpha)

    documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/DWIConvert

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Mark Scully (UIowa)

    acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.  Additional support for DTI data produced on Philips scanners was contributed by Vincent Magnotta and Hans Johnson at the University of Iowa.
    """

    input_spec = DWICompareInputSpec
    output_spec = DWICompareOutputSpec
    _cmd = " DWICompare "
    _outputs_filenames = {}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import CopyGeom


def test_CopyGeom_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        dest_file=dict(
            argstr="%s",
            copyfile=True,
            extensions=None,
            mandatory=True,
            name_source="dest_file",
            name_template="%s",
            output_name="out_file",
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ignore_dims=dict(
            argstr="-d",
            position="-1",
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=0,
        ),
        output_type=dict(),
    )
    inputs = CopyGeom.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_CopyGeom_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = CopyGeom.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import RealignUnwarp


def test_RealignUnwarp_inputs():
    input_map = dict(
        est_basis_func=dict(
            field="uweoptions.basfcn",
        ),
        est_first_order_effects=dict(
            field="uweoptions.fot",
        ),
        est_jacobian_deformations=dict(
            field="uweoptions.jm",
        ),
        est_num_of_iterations=dict(
            field="uweoptions.noi",
            maxlen=1,
            minlen=1,
            usedefault=True,
        ),
        est_re_est_mov_par=dict(
            field="uweoptions.rem",
        ),
        est_reg_factor=dict(
            field="uweoptions.lambda",
            maxlen=1,
            minlen=1,
            usedefault=True,
        ),
        est_reg_order=dict(
            field="uweoptions.regorder",
        ),
        est_second_order_effects=dict(
            field="uweoptions.sot",
        ),
        est_taylor_expansion_point=dict(
            field="uweoptions.expround",
            usedefault=True,
        ),
        est_unwarp_fwhm=dict(
            field="uweoptions.uwfwhm",
        ),
        fwhm=dict(
            field="eoptions.fwhm",
        ),
        in_files=dict(
            copyfile=True,
            field="data.scans",
            mandatory=True,
        ),
        interp=dict(
            field="eoptions.einterp",
        ),
        matlab_cmd=dict(),
        mfile=dict(
            usedefault=True,
        ),
        out_prefix=dict(
            field="uwroptions.prefix",
            usedefault=True,
        ),
        paths=dict(),
        phase_map=dict(
            copyfile=False,
            extensions=None,
            field="data.pmscan",
        ),
        quality=dict(
            field="eoptions.quality",
        ),
        register_to_mean=dict(
            field="eoptions.rtm",
        ),
        reslice_interp=dict(
            field="uwroptions.rinterp",
        ),
        reslice_mask=dict(
            field="uwroptions.mask",
        ),
        reslice_which=dict(
            field="uwroptions.uwwhich",
            maxlen=2,
            minlen=2,
            usedefault=True,
        ),
        reslice_wrap=dict(
            field="uwroptions.wrap",
        ),
        separation=dict(
            field="eoptions.sep",
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver="8",
            usedefault=True,
        ),
        weight_img=dict(
            extensions=None,
            field="eoptions.weight",
        ),
        wrap=dict(
            field="eoptions.ewrap",
        ),
    )
    inputs = RealignUnwarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_RealignUnwarp_outputs():
    output_map = dict(
        mean_image=dict(
            extensions=None,
        ),
        modified_in_files=dict(),
        realigned_unwarped_files=dict(),
        realignment_parameters=dict(),
    )
    outputs = RealignUnwarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..arithmetic import CastScalarVolume


def test_CastScalarVolume_inputs():
    input_map = dict(
        InputVolume=dict(
            argstr="%s",
            extensions=None,
            position=-2,
        ),
        OutputVolume=dict(
            argstr="%s",
            hash_files=False,
            position=-1,
        ),
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        type=dict(
            argstr="--type %s",
        ),
    )
    inputs = CastScalarVolume.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_CastScalarVolume_outputs():
    output_map = dict(
        OutputVolume=dict(
            extensions=None,
            position=-1,
        ),
    )
    outputs = CastScalarVolume.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

import numpy.testing as npt
from .. import rapidart as ra
from ...interfaces.base import Bunch


def test_ad_init():
    ad = ra.ArtifactDetect(use_differences=[True, False])
    assert ad.inputs.use_differences[0]
    assert not ad.inputs.use_differences[1]


def test_ad_output_filenames():
    ad = ra.ArtifactDetect()
    outputdir = "/tmp"
    f = "motion.nii"
    (
        outlierfile,
        intensityfile,
        statsfile,
        normfile,
        plotfile,
        displacementfile,
        maskfile,
    ) = ad._get_output_filenames(f, outputdir)
    assert outlierfile == "/tmp/art.motion_outliers.txt"
    assert intensityfile == "/tmp/global_intensity.motion.txt"
    assert statsfile == "/tmp/stats.motion.txt"
    assert normfile == "/tmp/norm.motion.txt"
    assert plotfile == "/tmp/plot.motion.png"
    assert displacementfile == "/tmp/disp.motion.nii"
    assert maskfile == "/tmp/mask.motion.nii"


def test_ad_get_affine_matrix():
    matrix = ra._get_affine_matrix(np.array([0]), "SPM")
    npt.assert_equal(matrix, np.eye(4))
    # test translation
    params = [1, 2, 3]
    matrix = ra._get_affine_matrix(params, "SPM")
    out = np.eye(4)
    out[0:3, 3] = params
    npt.assert_equal(matrix, out)
    # test rotation
    params = np.array([0, 0, 0, np.pi / 2, np.pi / 2, np.pi / 2])
    matrix = ra._get_affine_matrix(params, "SPM")
    out = np.array([0, 0, 1, 0, 0, -1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1]).reshape((4, 4))
    npt.assert_almost_equal(matrix, out)
    # test scaling
    params = np.array([0, 0, 0, 0, 0, 0, 1, 2, 3])
    matrix = ra._get_affine_matrix(params, "SPM")
    out = np.array([1, 0, 0, 0, 0, 2, 0, 0, 0, 0, 3, 0, 0, 0, 0, 1]).reshape((4, 4))
    npt.assert_equal(matrix, out)
    # test shear
    params = np.array([0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 3])
    matrix = ra._get_affine_matrix(params, "SPM")
    out = np.array([1, 1, 2, 0, 0, 1, 3, 0, 0, 0, 1, 0, 0, 0, 0, 1]).reshape((4, 4))
    npt.assert_equal(matrix, out)


def test_ad_get_norm():
    params = np.array(
        [
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            np.pi / 4,
            np.pi / 4,
            np.pi / 4,
            0,
            0,
            0,
            -np.pi / 4,
            -np.pi / 4,
            -np.pi / 4,
        ]
    ).reshape((3, 6))
    norm, _ = ra._calc_norm(params, False, "SPM")
    npt.assert_almost_equal(norm, np.array([18.86436316, 37.74610158, 31.29780829]))
    norm, _ = ra._calc_norm(params, True, "SPM")
    npt.assert_almost_equal(norm, np.array([0.0, 143.72192614, 173.92527131]))


def test_sc_init():
    sc = ra.StimulusCorrelation(concatenated_design=True)
    assert sc.inputs.concatenated_design


def test_sc_populate_inputs():
    sc = ra.StimulusCorrelation()
    inputs = Bunch(
        realignment_parameters=None,
        intensity_values=None,
        spm_mat_file=None,
        concatenated_design=None,
    )
    assert set(sc.inputs.__dict__.keys()) == set(inputs.__dict__.keys())


def test_sc_output_filenames():
    sc = ra.StimulusCorrelation()
    outputdir = "/tmp"
    f = "motion.nii"
    corrfile = sc._get_output_filenames(f, outputdir)
    assert corrfile == "/tmp/qa.motion_stimcorr.txt"



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import Synthesize


def test_Synthesize_inputs():
    input_map = dict(
        TR=dict(
            argstr="-TR %f",
        ),
        args=dict(
            argstr="%s",
        ),
        cbucket=dict(
            argstr="-cbucket %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        cenfill=dict(
            argstr="-cenfill %s",
        ),
        dry_run=dict(
            argstr="-dry",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        matrix=dict(
            argstr="-matrix %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_template="syn",
        ),
        outputtype=dict(),
        select=dict(
            argstr="-select %s",
            mandatory=True,
        ),
    )
    inputs = Synthesize.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Synthesize_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Synthesize.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Provides interfaces to various commands for running PET analyses provided by FreeSurfer
"""

import os

from ... import logging
from ..base import (
    TraitedSpec,
    File,
    traits,
    Directory,
    InputMultiPath,
    isdefined,
)
from .base import FSCommand, FSTraitedSpec

from .model import GLMFitInputSpec, GLMFit

__docformat__ = "restructuredtext"
iflogger = logging.getLogger("nipype.interface")


class GTMSegInputSpec(FSTraitedSpec):

    subject_id = traits.String(argstr="--s %s", desc="subject id", mandatory=True)

    xcerseg = traits.Bool(
        argstr="--xcerseg",
        desc="run xcerebralseg on this subject to create apas+head.mgz",
    )

    out_file = File(
        "gtmseg.mgz",
        argstr="--o %s",
        desc="output volume relative to subject/mri",
        usedefault=True,
    )

    upsampling_factor = traits.Int(
        argstr="--usf %i", desc="upsampling factor (default is 2)"
    )

    subsegwm = traits.Bool(
        argstr="--subsegwm", default=True, desc="subsegment WM into lobes (default)"
    )

    keep_hypo = traits.Bool(
        argstr="--keep-hypo",
        desc="do not relabel hypointensities as WM when subsegmenting WM",
    )

    keep_cc = traits.Bool(
        argstr="--keep-cc", desc="do not relabel corpus callosum as WM"
    )

    dmax = traits.Float(
        argstr="--dmax %f",
        desc="distance threshold to use when subsegmenting WM (default is 5)",
    )

    ctx_annot = traits.Tuple(
        traits.String,
        traits.Int,
        traits.Int,
        argstr="--ctx-annot %s %i %i",
        desc="annot lhbase rhbase : annotation to use for cortical segmentation (default is aparc 1000 2000)",
    )

    wm_annot = traits.Tuple(
        traits.String,
        traits.Int,
        traits.Int,
        argstr="--wm-annot %s %i %i",
        desc="annot lhbase rhbase : annotation to use for WM segmentation (with --subsegwm, default is lobes 3200 4200)",
    )

    output_upsampling_factor = traits.Int(
        argstr="--output-usf %i",
        desc="set output USF different than USF, mostly for debugging",
    )

    head = traits.String(
        argstr="--head %s", desc="use headseg instead of apas+head.mgz"
    )

    subseg_cblum_wm = traits.Bool(
        argstr="--subseg-cblum-wm", desc="subsegment cerebellum WM into core and gyri"
    )

    no_pons = traits.Bool(
        argstr="--no-pons", desc="do not add pons segmentation when doing ---xcerseg"
    )

    no_vermis = traits.Bool(
        argstr="--no-vermis",
        desc="do not add vermis segmentation when doing ---xcerseg",
    )

    colortable = File(exists=True, argstr="--ctab %s", desc="colortable")
    no_seg_stats = traits.Bool(
        argstr="--no-seg-stats", desc="do not compute segmentation stats"
    )


class GTMSegOutputSpec(TraitedSpec):
    out_file = File(desc="GTM segmentation")


class GTMSeg(FSCommand):
    """create an anatomical segmentation for the geometric transfer matrix (GTM).

    Examples
    --------
    >>> gtmseg = GTMSeg()
    >>> gtmseg.inputs.subject_id = 'subject_id'
    >>> gtmseg.cmdline
    'gtmseg --o gtmseg.mgz --s subject_id'
    """

    _cmd = "gtmseg"
    input_spec = GTMSegInputSpec
    output_spec = GTMSegOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.join(
            self.inputs.subjects_dir,
            self.inputs.subject_id,
            'mri',
            self.inputs.out_file,
        )
        return outputs


class GTMPVCInputSpec(FSTraitedSpec):

    in_file = File(
        exists=True,
        argstr="--i %s",
        mandatory=True,
        copyfile=False,
        desc="input volume - source data to pvc",
    )

    frame = traits.Int(
        argstr="--frame %i", desc="only process 0-based frame F from inputvol"
    )

    psf = traits.Float(argstr="--psf %f", desc="scanner PSF FWHM in mm")

    segmentation = File(
        argstr="--seg %s",
        exists=True,
        mandatory=True,
        desc="segfile : anatomical segmentation to define regions for GTM",
    )

    _reg_xor = ["reg_file", "regheader", "reg_identity"]
    reg_file = File(
        exists=True,
        argstr="--reg %s",
        mandatory=True,
        desc="LTA registration file that maps PET to anatomical",
        xor=_reg_xor,
    )

    regheader = traits.Bool(
        argstr="--regheader",
        mandatory=True,
        desc="assume input and seg share scanner space",
        xor=_reg_xor,
    )

    reg_identity = traits.Bool(
        argstr="--reg-identity",
        mandatory=True,
        desc="assume that input is in anatomical space",
        xor=_reg_xor,
    )

    pvc_dir = traits.Str(argstr="--o %s", desc="save outputs to dir", genfile=True)

    mask_file = File(
        exists=True,
        argstr="--mask %s",
        desc="ignore areas outside of the mask (in input vol space)",
    )

    auto_mask = traits.Tuple(
        traits.Float,
        traits.Float,
        argstr="--auto-mask %f %f",
        desc="FWHM thresh : automatically compute mask",
    )

    no_reduce_fov = traits.Bool(
        argstr="--no-reduce-fov", desc="do not reduce FoV to encompass mask"
    )

    reduce_fox_eqodd = traits.Bool(
        argstr="--reduce-fox-eqodd",
        desc="reduce FoV to encompass mask but force nc=nr and ns to be odd",
    )

    contrast = InputMultiPath(
        File(exists=True), argstr="--C %s...", desc="contrast file"
    )

    default_seg_merge = traits.Bool(
        argstr="--default-seg-merge", desc="default schema for merging ROIs"
    )

    merge_hypos = traits.Bool(
        argstr="--merge-hypos", desc="merge left and right hypointensites into to ROI"
    )

    merge_cblum_wm_gyri = traits.Bool(
        argstr="--merge-cblum-wm-gyri",
        desc="cerebellum WM gyri back into cerebellum WM",
    )

    tt_reduce = traits.Bool(
        argstr="--tt-reduce", desc="reduce segmentation to that of a tissue type"
    )

    replace = traits.Tuple(
        traits.Int,
        traits.Int,
        argstr="--replace %i %i",
        desc="Id1 Id2 : replace seg Id1 with seg Id2",
    )

    rescale = traits.List(
        argstr="--rescale %s...",
        desc="Id1 <Id2...>  : specify reference region(s) used to rescale (default is pons)",
    )

    no_rescale = traits.Bool(
        argstr="--no-rescale",
        desc="do not global rescale such that mean of reference region is scaleref",
    )

    scale_refval = traits.Float(
        argstr="--scale-refval %f",
        desc="refval : scale such that mean in reference region is refval",
    )

    _ctab_inputs = ("color_table_file", "default_color_table")
    color_table_file = File(
        exists=True,
        argstr="--ctab %s",
        xor=_ctab_inputs,
        desc="color table file with seg id names",
    )

    default_color_table = traits.Bool(
        argstr="--ctab-default",
        xor=_ctab_inputs,
        desc="use $FREESURFER_HOME/FreeSurferColorLUT.txt",
    )

    tt_update = traits.Bool(
        argstr="--tt-update",
        desc="changes tissue type of VentralDC, BrainStem, and Pons to be SubcortGM",
    )

    lat = traits.Bool(argstr="--lat", desc="lateralize tissue types")

    no_tfe = traits.Bool(
        argstr="--no-tfe",
        desc="do not correct for tissue fraction effect (with --psf 0 turns off PVC entirely)",
    )

    no_pvc = traits.Bool(
        argstr="--no-pvc",
        desc="turns off PVC entirely (both PSF and TFE)",
    )

    tissue_fraction_resolution = traits.Float(
        argstr="--segpvfres %f",
        desc="set the tissue fraction resolution parameter (def is 0.5)",
    )

    rbv = traits.Bool(
        argstr="--rbv",
        requires=["subjects_dir"],
        desc="perform Region-based Voxelwise (RBV) PVC",
    )

    rbv_res = traits.Float(
        argstr="--rbv-res %f",
        desc="voxsize : set RBV voxel resolution (good for when standard res takes too much memory)",
    )

    mg = traits.Tuple(
        traits.Float,
        traits.List(traits.String),
        argstr="--mg %g %s",
        desc="gmthresh RefId1 RefId2 ...: perform Mueller-Gaertner PVC, gmthresh is min gm pvf bet 0 and 1",
    )

    mg_ref_cerebral_wm = traits.Bool(
        argstr="--mg-ref-cerebral-wm", desc=" set MG RefIds to 2 and 41"
    )

    mg_ref_lobes_wm = traits.Bool(
        argstr="--mg-ref-lobes-wm",
        desc="set MG RefIds to those for lobes when using wm subseg",
    )

    mgx = traits.Float(
        argstr="--mgx %f",
        desc="gmxthresh : GLM-based Mueller-Gaertner PVC, gmxthresh is min gm pvf bet 0 and 1",
    )

    km_ref = traits.List(
        argstr="--km-ref %s...",
        desc="RefId1 RefId2 ... : compute reference TAC for KM as mean of given RefIds",
    )

    km_hb = traits.List(
        argstr="--km-hb %s...",
        desc="RefId1 RefId2 ... : compute HiBinding TAC for KM as mean of given RefIds",
    )

    steady_state_params = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        argstr="--ss %f %f %f",
        desc="bpc scale dcf : steady-state analysis spec blood plasma concentration, unit scale and decay correction factor. You must also spec --km-ref. Turns off rescaling",
    )

    X = traits.Bool(
        argstr="--X", desc="save X matrix in matlab4 format as X.mat (it will be big)"
    )

    y = traits.Bool(argstr="--y", desc="save y matrix in matlab4 format as y.mat")

    beta = traits.Bool(
        argstr="--beta", desc="save beta matrix in matlab4 format as beta.mat"
    )

    X0 = traits.Bool(
        argstr="--X0",
        desc="save X0 matrix in matlab4 format as X0.mat (it will be big)",
    )

    save_input = traits.Bool(
        argstr="--save-input", desc="saves rescaled input as input.rescaled.nii.gz"
    )

    save_eres = traits.Bool(argstr="--save-eres", desc="saves residual error")

    save_yhat = traits.Bool(
        argstr="--save-yhat",
        xor=["save_yhat_with_noise"],
        desc="save signal estimate (yhat) smoothed with the PSF",
    )

    save_yhat_with_noise = traits.Tuple(
        traits.Int,
        traits.Int,
        argstr="--save-yhat-with-noise %i %i",
        xor=["save_yhat"],
        desc="seed nreps : save signal estimate (yhat) with noise",
    )

    save_yhat_full_fov = traits.Bool(
        argstr="--save-yhat-full-fov", desc="save signal estimate (yhat)"
    )

    save_yhat0 = traits.Bool(argstr="--save-yhat0", desc="save signal estimate (yhat)")

    optimization_schema = traits.Enum(
        "3D",
        "2D",
        "1D",
        "3D_MB",
        "2D_MB",
        "1D_MB",
        "MBZ",
        "MB3",
        argstr="--opt %s",
        desc="opt : optimization schema for applying adaptive GTM",
    )

    opt_tol = traits.Tuple(
        traits.Int,
        traits.Float,
        traits.Float,
        argstr="--opt-tol %i %f %f",
        desc="n_iters_max ftol lin_min_tol : optimization parameters for adaptive gtm using fminsearch",
    )

    opt_brain = traits.Bool(argstr="--opt-brain", desc="apply adaptive GTM")

    opt_seg_merge = traits.Bool(
        argstr="--opt-seg-merge",
        desc="optimal schema for merging ROIs when applying adaptive GTM",
    )

    num_threads = traits.Int(
        argstr="--threads %i", desc="threads : number of threads to use"
    )

    psf_col = traits.Float(
        argstr="--psf-col %f", desc="xFWHM : full-width-half-maximum in the x-direction"
    )

    psf_row = traits.Float(
        argstr="--psf-row %f", desc="yFWHM : full-width-half-maximum in the y-direction"
    )

    psf_slice = traits.Float(
        argstr="--psf-slice %f",
        desc="zFWHM : full-width-half-maximum in the z-direction",
    )


class GTMPVCOutputSpec(TraitedSpec):

    pvc_dir = Directory(desc="output directory")
    ref_file = File(desc="Reference TAC in .dat")
    hb_nifti = File(desc="High-binding TAC in nifti")
    hb_dat = File(desc="High-binding TAC in .dat")
    nopvc_file = File(desc="TACs for all regions with no PVC")
    gtm_file = File(desc="TACs for all regions with GTM PVC")
    gtm_stats = File(desc="Statistics for the GTM PVC")
    input_file = File(desc="4D PET file in native volume space")
    reg_pet2anat = File(desc="Registration file to go from PET to anat")
    reg_anat2pet = File(desc="Registration file to go from anat to PET")
    reg_rbvpet2anat = File(
        desc="Registration file to go from RBV corrected PET to anat"
    )
    reg_anat2rbvpet = File(
        desc="Registration file to go from anat to RBV corrected PET"
    )
    mgx_ctxgm = File(
        desc="Cortical GM voxel-wise values corrected using the extended Muller-Gartner method",
    )
    mgx_subctxgm = File(
        desc="Subcortical GM voxel-wise values corrected using the extended Muller-Gartner method",
    )
    mgx_gm = File(
        desc="All GM voxel-wise values corrected using the extended Muller-Gartner method",
    )
    rbv = File(desc="All GM voxel-wise values corrected using the RBV method")
    opt_params = File(
        desc="Optimal parameter estimates for the FWHM using adaptive GTM"
    )
    yhat0 = File(desc="4D PET file of signal estimate (yhat) after PVC (unsmoothed)")
    yhat = File(
        desc="4D PET file of signal estimate (yhat) after PVC (smoothed with PSF)",
    )
    yhat_full_fov = File(
        desc="4D PET file with full FOV of signal estimate (yhat) after PVC (smoothed with PSF)",
    )
    yhat_with_noise = File(
        desc="4D PET file with full FOV of signal estimate (yhat) with noise after PVC (smoothed with PSF)",
    )


class GTMPVC(FSCommand):
    """create an anatomical segmentation for the geometric transfer matrix (GTM).

    Examples
    --------
    >>> gtmpvc = GTMPVC()
    >>> gtmpvc.inputs.in_file = 'sub-01_ses-baseline_pet.nii.gz'
    >>> gtmpvc.inputs.segmentation = 'gtmseg.mgz'
    >>> gtmpvc.inputs.reg_file = 'sub-01_ses-baseline_pet_mean_reg.lta'
    >>> gtmpvc.inputs.pvc_dir = 'pvc'
    >>> gtmpvc.inputs.psf = 4
    >>> gtmpvc.inputs.default_seg_merge = True
    >>> gtmpvc.inputs.auto_mask = (1, 0.1)
    >>> gtmpvc.inputs.km_ref = ['8 47']
    >>> gtmpvc.inputs.km_hb = ['11 12 50 51']
    >>> gtmpvc.inputs.no_rescale = True
    >>> gtmpvc.inputs.save_input = True
    >>> gtmpvc.cmdline  # doctest: +NORMALIZE_WHITESPACE
    'mri_gtmpvc --auto-mask 1.000000 0.100000 --default-seg-merge \
    --i sub-01_ses-baseline_pet.nii.gz --km-hb 11 12 50 51 --km-ref 8 47 --no-rescale \
    --psf 4.000000 --o pvc --reg sub-01_ses-baseline_pet_mean_reg.lta --save-input \
    --seg gtmseg.mgz'

    >>> gtmpvc = GTMPVC()
    >>> gtmpvc.inputs.in_file = 'sub-01_ses-baseline_pet.nii.gz'
    >>> gtmpvc.inputs.segmentation = 'gtmseg.mgz'
    >>> gtmpvc.inputs.regheader = True
    >>> gtmpvc.inputs.pvc_dir = 'pvc'
    >>> gtmpvc.inputs.mg = (0.5, ["ROI1", "ROI2"])
    >>> gtmpvc.cmdline  # doctest: +NORMALIZE_WHITESPACE
    'mri_gtmpvc --i sub-01_ses-baseline_pet.nii.gz --mg 0.5 ROI1 ROI2 --o pvc --regheader --seg gtmseg.mgz'
    """

    _cmd = "mri_gtmpvc"
    input_spec = GTMPVCInputSpec
    output_spec = GTMPVCOutputSpec

    def _format_arg(self, name, spec, val):
        # Values taken from
        # https://github.com/freesurfer/freesurfer/blob/fs-7.2/mri_gtmpvc/mri_gtmpvc.cpp#L115-L122
        if name == 'optimization_schema':
            return (
                spec.argstr
                % {
                    "3D": 1,
                    "2D": 2,
                    "1D": 3,
                    "3D_MB": 4,
                    "2D_MB": 5,
                    "1D_MB": 6,
                    "MBZ": 7,
                    "MB3": 8,
                }[val]
            )
        if name == 'mg':
            return spec.argstr % (val[0], ' '.join(val[1]))
        return super(GTMPVC, self)._format_arg(name, spec, val)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        # Get the top-level output directory
        if not isdefined(self.inputs.pvc_dir):
            pvcdir = os.getcwd()
        else:
            pvcdir = os.path.abspath(self.inputs.pvc_dir)
        outputs["pvc_dir"] = pvcdir

        # Assign the output files that always get created
        outputs["ref_file"] = os.path.join(pvcdir, "km.ref.tac.dat")
        outputs["hb_nifti"] = os.path.join(pvcdir, "km.hb.tac.nii.gz")
        outputs["hb_dat"] = os.path.join(pvcdir, "km.hb.tac.dat")
        outputs["nopvc_file"] = os.path.join(pvcdir, "nopvc.nii.gz")
        outputs["gtm_file"] = os.path.join(pvcdir, "gtm.nii.gz")
        outputs["gtm_stats"] = os.path.join(pvcdir, "gtm.stats.dat")
        outputs["reg_pet2anat"] = os.path.join(pvcdir, "aux", "bbpet2anat.lta")
        outputs["reg_anat2pet"] = os.path.join(pvcdir, "aux", "anat2bbpet.lta")

        # Assign the conditional outputs
        if self.inputs.save_input:
            outputs["input_file"] = os.path.join(pvcdir, "input.nii.gz")
        if self.inputs.save_yhat0:
            outputs["yhat0"] = os.path.join(pvcdir, "yhat0.nii.gz")
        if self.inputs.save_yhat:
            outputs["yhat"] = os.path.join(pvcdir, "yhat.nii.gz")
        if self.inputs.save_yhat_full_fov:
            outputs["yhat_full_fov"] = os.path.join(pvcdir, "yhat.fullfov.nii.gz")
        if self.inputs.save_yhat_with_noise:
            outputs["yhat_with_noise"] = os.path.join(pvcdir, "yhat.nii.gz")
        if self.inputs.mgx:
            outputs["mgx_ctxgm"] = os.path.join(pvcdir, "mgx.ctxgm.nii.gz")
            outputs["mgx_subctxgm"] = os.path.join(pvcdir, "mgx.subctxgm.nii.gz")
            outputs["mgx_gm"] = os.path.join(pvcdir, "mgx.gm.nii.gz")
        if self.inputs.rbv:
            outputs["rbv"] = os.path.join(pvcdir, "rbv.nii.gz")
            outputs["reg_rbvpet2anat"] = os.path.join(pvcdir, "aux", "rbv2anat.lta")
            outputs["reg_anat2rbvpet"] = os.path.join(pvcdir, "aux", "anat2rbv.lta")
        if self.inputs.opt:
            outputs["opt_params"] = os.path.join(pvcdir, "aux", "opt.params.dat")

        return outputs


class MRTMInputSpec(GLMFitInputSpec):
    mrtm1 = traits.Tuple(
        File(exists=True),
        File(exists=True),
        mandatory=True,
        argstr="--mrtm1 %s %s",
        desc="RefTac TimeSec : perform MRTM1 kinetic modeling",
    )


class MRTM(GLMFit):
    """Perform MRTM1 kinetic modeling.

    Examples
    --------
    >>> mrtm = MRTM()
    >>> mrtm.inputs.in_file = 'tac.nii'
    >>> mrtm.inputs.mrtm1 = ('ref_tac.dat', 'timing.dat')
    >>> mrtm.inputs.glm_dir = 'mrtm'
    >>> mrtm.cmdline
    'mri_glmfit --glmdir mrtm --y tac.nii --mrtm1 ref_tac.dat timing.dat'
    """

    input_spec = MRTMInputSpec


class MRTM2InputSpec(GLMFitInputSpec):
    mrtm2 = traits.Tuple(
        File(exists=True),
        File(exists=True),
        traits.Float,
        mandatory=True,
        argstr="--mrtm2 %s %s %f",
        desc="RefTac TimeSec k2prime : perform MRTM2 kinetic modeling",
    )


class MRTM2(GLMFit):
    """Perform MRTM2 kinetic modeling.
    Examples
    --------
    >>> mrtm2 = MRTM2()
    >>> mrtm2.inputs.in_file = 'tac.nii'
    >>> mrtm2.inputs.mrtm2 = ('ref_tac.dat', 'timing.dat', 0.07872)
    >>> mrtm2.inputs.glm_dir = 'mrtm2'
    >>> mrtm2.cmdline
    'mri_glmfit --glmdir mrtm2 --y tac.nii --mrtm2 ref_tac.dat timing.dat 0.078720'
    """

    input_spec = MRTM2InputSpec


class LoganRefInputSpec(GLMFitInputSpec):
    logan = traits.Tuple(
        File(exists=True),
        File(exists=True),
        traits.Float,
        mandatory=True,
        argstr="--logan %s %s %g",
        desc="RefTac TimeSec tstar   : perform Logan kinetic modeling",
    )


class LoganRef(GLMFit):
    """Perform Logan reference kinetic modeling.
    Examples
    --------
    >>> logan = LoganRef()
    >>> logan.inputs.in_file = 'tac.nii'
    >>> logan.inputs.logan = ('ref_tac.dat', 'timing.dat', 2600)
    >>> logan.inputs.glm_dir = 'logan'
    >>> logan.cmdline
    'mri_glmfit --glmdir logan --y tac.nii --logan ref_tac.dat timing.dat 2600'
    """

    input_spec = LoganRefInputSpec



#!/usr/bin/env python
import pytest
import sys
from contextlib import contextmanager

from io import StringIO
from ...utils import nipype_cmd


@contextmanager
def capture_sys_output():
    caputure_out, capture_err = StringIO(), StringIO()
    current_out, current_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = caputure_out, capture_err
        yield caputure_out, capture_err
    finally:
        sys.stdout, sys.stderr = current_out, current_err


class TestNipypeCMD:
    maxDiff = None

    def test_main_returns_2_on_empty(self):
        with pytest.raises(SystemExit) as cm:
            with capture_sys_output() as (stdout, stderr):
                nipype_cmd.main(["nipype_cmd"])

        exit_exception = cm.value
        assert exit_exception.code == 2

        msg = """usage: nipype_cmd [-h] module interface
nipype_cmd: error: the following arguments are required: module, interface
"""

        assert stderr.getvalue() == msg
        assert stdout.getvalue() == ""

    def test_main_returns_0_on_help(self):
        with pytest.raises(SystemExit) as cm:
            with capture_sys_output() as (stdout, stderr):
                nipype_cmd.main(["nipype_cmd", "-h"])

        exit_exception = cm.value
        assert exit_exception.code == 0

        assert stderr.getvalue() == ""
        if sys.version_info >= (3, 10):
            options = "options"
        else:
            options = "optional arguments"
        assert (
            stdout.getvalue()
            == f"""usage: nipype_cmd [-h] module interface

Nipype interface runner

positional arguments:
  module      Module name
  interface   Interface name

{options}:
  -h, --help  show this help message and exit
"""
        )

    def test_list_nipy_interfacesp(self):
        with pytest.raises(SystemExit) as cm:
            with capture_sys_output() as (stdout, stderr):
                nipype_cmd.main(["nipype_cmd", "nipype.interfaces.nipy"])

        # repeat twice in case nipy raises warnings
        with pytest.raises(SystemExit) as cm:
            with capture_sys_output() as (stdout, stderr):
                nipype_cmd.main(["nipype_cmd", "nipype.interfaces.nipy"])
        exit_exception = cm.value
        assert exit_exception.code == 0

        assert stderr.getvalue() == ""
        assert (
            stdout.getvalue()
            == """Available Interfaces:
\tComputeMask
\tEstimateContrast
\tFitGLM
\tSimilarity
\tSpaceTimeRealigner
"""
        )



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import ReconAll


def test_ReconAll_inputs():
    input_map = dict(
        FLAIR_file=dict(
            argstr="-FLAIR %s",
            extensions=None,
            min_ver="5.3.0",
        ),
        T1_files=dict(
            argstr="-i %s...",
        ),
        T2_file=dict(
            argstr="-T2 %s",
            extensions=None,
            min_ver="5.3.0",
        ),
        args=dict(
            argstr="%s",
        ),
        big_ventricles=dict(
            argstr="-bigventricles",
        ),
        brainstem=dict(
            argstr="-brainstem-structures",
        ),
        directive=dict(
            argstr="-%s",
            position=0,
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expert=dict(
            argstr="-expert %s",
            extensions=None,
        ),
        flags=dict(
            argstr="%s",
        ),
        hemi=dict(
            argstr="-hemi %s",
        ),
        hippocampal_subfields_T1=dict(
            argstr="-hippocampal-subfields-T1",
            min_ver="6.0.0",
        ),
        hippocampal_subfields_T2=dict(
            argstr="-hippocampal-subfields-T2 %s %s",
            min_ver="6.0.0",
        ),
        hires=dict(
            argstr="-hires",
            min_ver="6.0.0",
        ),
        mprage=dict(
            argstr="-mprage",
        ),
        mri_aparc2aseg=dict(
            xor=["expert"],
        ),
        mri_ca_label=dict(
            xor=["expert"],
        ),
        mri_ca_normalize=dict(
            xor=["expert"],
        ),
        mri_ca_register=dict(
            xor=["expert"],
        ),
        mri_edit_wm_with_aseg=dict(
            xor=["expert"],
        ),
        mri_em_register=dict(
            xor=["expert"],
        ),
        mri_fill=dict(
            xor=["expert"],
        ),
        mri_mask=dict(
            xor=["expert"],
        ),
        mri_normalize=dict(
            xor=["expert"],
        ),
        mri_pretess=dict(
            xor=["expert"],
        ),
        mri_remove_neck=dict(
            xor=["expert"],
        ),
        mri_segment=dict(
            xor=["expert"],
        ),
        mri_segstats=dict(
            xor=["expert"],
        ),
        mri_tessellate=dict(
            xor=["expert"],
        ),
        mri_watershed=dict(
            xor=["expert"],
        ),
        mris_anatomical_stats=dict(
            xor=["expert"],
        ),
        mris_ca_label=dict(
            xor=["expert"],
        ),
        mris_fix_topology=dict(
            xor=["expert"],
        ),
        mris_inflate=dict(
            xor=["expert"],
        ),
        mris_make_surfaces=dict(
            xor=["expert"],
        ),
        mris_register=dict(
            xor=["expert"],
        ),
        mris_smooth=dict(
            xor=["expert"],
        ),
        mris_sphere=dict(
            xor=["expert"],
        ),
        mris_surf2vol=dict(
            xor=["expert"],
        ),
        mrisp_paint=dict(
            xor=["expert"],
        ),
        openmp=dict(
            argstr="-openmp %d",
        ),
        parallel=dict(
            argstr="-parallel",
        ),
        subject_id=dict(
            argstr="-subjid %s",
            usedefault=True,
        ),
        subjects_dir=dict(
            argstr="-sd %s",
            genfile=True,
            hash_files=False,
        ),
        talairach=dict(
            xor=["expert"],
        ),
        use_FLAIR=dict(
            argstr="-FLAIRpial",
            min_ver="5.3.0",
            xor=["use_T2"],
        ),
        use_T2=dict(
            argstr="-T2pial",
            min_ver="5.3.0",
            xor=["use_FLAIR"],
        ),
        xopts=dict(
            argstr="-xopts-%s",
        ),
    )
    inputs = ReconAll.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ReconAll_outputs():
    output_map = dict(
        BA_stats=dict(
            altkey="BA",
            loc="stats",
        ),
        T1=dict(
            extensions=None,
            loc="mri",
        ),
        annot=dict(
            altkey="*annot",
            loc="label",
        ),
        aparc_a2009s_stats=dict(
            altkey="aparc.a2009s",
            loc="stats",
        ),
        aparc_aseg=dict(
            altkey="aparc*aseg",
            loc="mri",
        ),
        aparc_stats=dict(
            altkey="aparc",
            loc="stats",
        ),
        area_pial=dict(
            altkey="area.pial",
            loc="surf",
        ),
        aseg=dict(
            extensions=None,
            loc="mri",
        ),
        aseg_stats=dict(
            altkey="aseg",
            loc="stats",
        ),
        avg_curv=dict(
            loc="surf",
        ),
        brain=dict(
            extensions=None,
            loc="mri",
        ),
        brainmask=dict(
            extensions=None,
            loc="mri",
        ),
        curv=dict(
            loc="surf",
        ),
        curv_pial=dict(
            altkey="curv.pial",
            loc="surf",
        ),
        curv_stats=dict(
            altkey="curv",
            loc="stats",
        ),
        entorhinal_exvivo_stats=dict(
            altkey="entorhinal_exvivo",
            loc="stats",
        ),
        filled=dict(
            extensions=None,
            loc="mri",
        ),
        graymid=dict(
            altkey=["graymid", "midthickness"],
            loc="surf",
        ),
        inflated=dict(
            loc="surf",
        ),
        jacobian_white=dict(
            loc="surf",
        ),
        label=dict(
            altkey="*label",
            loc="label",
        ),
        norm=dict(
            extensions=None,
            loc="mri",
        ),
        nu=dict(
            extensions=None,
            loc="mri",
        ),
        orig=dict(
            extensions=None,
            loc="mri",
        ),
        pial=dict(
            loc="surf",
        ),
        rawavg=dict(
            extensions=None,
            loc="mri",
        ),
        ribbon=dict(
            altkey="*ribbon",
            loc="mri",
        ),
        smoothwm=dict(
            loc="surf",
        ),
        sphere=dict(
            loc="surf",
        ),
        sphere_reg=dict(
            altkey="sphere.reg",
            loc="surf",
        ),
        subject_id=dict(),
        subjects_dir=dict(),
        sulc=dict(
            loc="surf",
        ),
        thickness=dict(
            loc="surf",
        ),
        volume=dict(
            loc="surf",
        ),
        white=dict(
            loc="surf",
        ),
        wm=dict(
            extensions=None,
            loc="mri",
        ),
        wmparc=dict(
            extensions=None,
            loc="mri",
        ),
        wmparc_stats=dict(
            altkey="wmparc",
            loc="stats",
        ),
    )
    outputs = ReconAll.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..minc import ToRaw


def test_ToRaw_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        nonormalize=dict(
            argstr="-nonormalize",
            xor=("normalize", "nonormalize"),
        ),
        normalize=dict(
            argstr="-normalize",
            xor=("normalize", "nonormalize"),
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        output_file=dict(
            extensions=None,
            hash_files=False,
            keep_extension=False,
            name_source=["input_file"],
            name_template="%s.raw",
            position=-1,
        ),
        write_byte=dict(
            argstr="-byte",
            xor=(
                "write_byte",
                "write_short",
                "write_int",
                "write_long",
                "write_float",
                "write_double",
            ),
        ),
        write_double=dict(
            argstr="-double",
            xor=(
                "write_byte",
                "write_short",
                "write_int",
                "write_long",
                "write_float",
                "write_double",
            ),
        ),
        write_float=dict(
            argstr="-float",
            xor=(
                "write_byte",
                "write_short",
                "write_int",
                "write_long",
                "write_float",
                "write_double",
            ),
        ),
        write_int=dict(
            argstr="-int",
            xor=(
                "write_byte",
                "write_short",
                "write_int",
                "write_long",
                "write_float",
                "write_double",
            ),
        ),
        write_long=dict(
            argstr="-long",
            xor=(
                "write_byte",
                "write_short",
                "write_int",
                "write_long",
                "write_float",
                "write_double",
            ),
        ),
        write_range=dict(
            argstr="-range %s %s",
        ),
        write_short=dict(
            argstr="-short",
            xor=(
                "write_byte",
                "write_short",
                "write_int",
                "write_long",
                "write_float",
                "write_double",
            ),
        ),
        write_signed=dict(
            argstr="-signed",
            xor=("write_signed", "write_unsigned"),
        ),
        write_unsigned=dict(
            argstr="-unsigned",
            xor=("write_signed", "write_unsigned"),
        ),
    )
    inputs = ToRaw.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ToRaw_outputs():
    output_map = dict(
        output_file=dict(
            extensions=None,
        ),
    )
    outputs = ToRaw.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from shutil import which

import nipype.interfaces.base as nib
import pytest
import nipype.pipeline.engine as pe


class InputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(desc="a random int")
    input2 = nib.traits.Int(desc="a random int")


class OutputSpec(nib.TraitedSpec):
    output1 = nib.traits.List(nib.traits.Int, desc="outputs")


class PbsTestInterface(nib.BaseInterface):
    input_spec = InputSpec
    output_spec = OutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["output1"] = [1, self.inputs.input1]
        return outputs


@pytest.mark.skipif(which("qsub") is None, reason="PBS not installed")
@pytest.mark.timeout(60)
def test_run_pbsgraph(tmp_path):
    pipe = pe.Workflow(name="pipe", base_dir=str(tmp_path))
    mod1 = pe.Node(interface=PbsTestInterface(), name="mod1")
    mod2 = pe.MapNode(interface=PbsTestInterface(), iterfield=["input1"], name="mod2")
    pipe.connect([(mod1, mod2, [("output1", "input1")])])
    mod1.inputs.input1 = 1
    execgraph = pipe.run(plugin="PBSGraph")
    names = [".".join((node._hierarchy, node.name)) for node in execgraph.nodes()]
    node = list(execgraph.nodes())[names.index("pipe.mod1")]
    result = node.get_output("output1")
    assert result == [1, 1]



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import EstimateModel


def test_EstimateModel_inputs():
    input_map = dict(
        estimation_method=dict(
            field="method",
            mandatory=True,
        ),
        flags=dict(),
        matlab_cmd=dict(),
        mfile=dict(
            usedefault=True,
        ),
        paths=dict(),
        spm_mat_file=dict(
            copyfile=True,
            extensions=None,
            field="spmmat",
            mandatory=True,
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver="8",
            usedefault=True,
        ),
        write_residuals=dict(
            field="write_residuals",
        ),
    )
    inputs = EstimateModel.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_EstimateModel_outputs():
    output_map = dict(
        ARcoef=dict(),
        Cbetas=dict(),
        RPVimage=dict(
            extensions=[".hdr", ".img", ".img.gz", ".nii"],
        ),
        SDbetas=dict(),
        SDerror=dict(),
        beta_images=dict(),
        labels=dict(
            extensions=[".hdr", ".img", ".img.gz", ".nii"],
        ),
        mask_image=dict(
            extensions=[".hdr", ".img", ".img.gz", ".nii"],
        ),
        residual_image=dict(
            extensions=[".hdr", ".img", ".img.gz", ".nii"],
        ),
        residual_images=dict(),
        spm_mat_file=dict(
            extensions=None,
        ),
    )
    outputs = EstimateModel.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import DWIPreproc


def test_DWIPreproc_inputs():
    input_map = dict(
        align_seepi=dict(
            argstr="-align_seepi",
        ),
        args=dict(
            argstr="%s",
        ),
        bval_scale=dict(
            argstr="-bvalue_scaling %s",
        ),
        eddy_options=dict(
            argstr='-eddy_options "%s"',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        export_grad_fsl=dict(
            argstr="-export_grad_fsl",
        ),
        export_grad_mrtrix=dict(
            argstr="-export_grad_mrtrix",
        ),
        grad_file=dict(
            argstr="-grad %s",
            extensions=None,
            xor=["grad_fsl"],
        ),
        grad_fsl=dict(
            argstr="-fslgrad %s %s",
            xor=["grad_file"],
        ),
        in_bval=dict(
            extensions=None,
        ),
        in_bvec=dict(
            argstr="-fslgrad %s %s",
            extensions=None,
        ),
        in_epi=dict(
            argstr="-se_epi %s",
            extensions=None,
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=0,
        ),
        nthreads=dict(
            argstr="-nthreads %d",
            nohash=True,
        ),
        out_bval=dict(
            extensions=None,
        ),
        out_bvec=dict(
            argstr="-export_grad_fsl %s %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=1,
            usedefault=True,
        ),
        out_grad_fsl=dict(
            argstr="%s, %s",
            requires=["export_grad_fsl"],
        ),
        out_grad_mrtrix=dict(
            argstr="%s",
            extensions=None,
            requires=["export_grad_mrtrix"],
            usedefault=True,
        ),
        pe_dir=dict(
            argstr="-pe_dir %s",
            mandatory=True,
        ),
        ro_time=dict(
            argstr="-readout_time %f",
        ),
        rpe_options=dict(
            argstr="-rpe_%s",
            mandatory=True,
            position=2,
        ),
        topup_options=dict(
            argstr='-topup_options "%s"',
        ),
    )
    inputs = DWIPreproc.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_DWIPreproc_outputs():
    output_map = dict(
        out_file=dict(
            argstr="%s",
            extensions=None,
        ),
        out_fsl_bval=dict(
            argstr="%s",
            extensions=None,
            usedefault=True,
        ),
        out_fsl_bvec=dict(
            argstr="%s",
            extensions=None,
            usedefault=True,
        ),
        out_grad_mrtrix=dict(
            argstr="%s",
            extensions=None,
            usedefault=True,
        ),
    )
    outputs = DWIPreproc.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..brains import ImageRegionPlotter


def test_ImageRegionPlotter_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputBinaryROIVolume=dict(
            argstr="--inputBinaryROIVolume %s",
            extensions=None,
        ),
        inputLabelVolume=dict(
            argstr="--inputLabelVolume %s",
            extensions=None,
        ),
        inputVolume1=dict(
            argstr="--inputVolume1 %s",
            extensions=None,
        ),
        inputVolume2=dict(
            argstr="--inputVolume2 %s",
            extensions=None,
        ),
        numberOfHistogramBins=dict(
            argstr="--numberOfHistogramBins %d",
        ),
        outputJointHistogramData=dict(
            argstr="--outputJointHistogramData %s",
        ),
        useIntensityForHistogram=dict(
            argstr="--useIntensityForHistogram ",
        ),
        useROIAUTO=dict(
            argstr="--useROIAUTO ",
        ),
        verbose=dict(
            argstr="--verbose ",
        ),
    )
    inputs = ImageRegionPlotter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ImageRegionPlotter_outputs():
    output_map = dict()
    outputs = ImageRegionPlotter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)


class SimilarityIndexInputSpec(CommandLineInputSpec):
    outputCSVFilename = File(
        desc="output CSV Filename", exists=True, argstr="--outputCSVFilename %s"
    )
    ANNContinuousVolume = File(
        desc="ANN Continuous volume to be compared to the manual volume",
        exists=True,
        argstr="--ANNContinuousVolume %s",
    )
    inputManualVolume = File(
        desc="input manual(reference) volume",
        exists=True,
        argstr="--inputManualVolume %s",
    )
    thresholdInterval = traits.Float(
        desc="Threshold interval to compute similarity index between zero and one",
        argstr="--thresholdInterval %f",
    )


class SimilarityIndexOutputSpec(TraitedSpec):
    pass


class SimilarityIndex(SEMLikeCommandLine):
    """title: BRAINSCut:SimilarityIndexComputation

    category: BRAINS.Segmentation

    description: Automatic analysis of BRAINSCut Output

    version: 1.0

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Eunyoung Regin Kim
    """

    input_spec = SimilarityIndexInputSpec
    output_spec = SimilarityIndexOutputSpec
    _cmd = " SimilarityIndex "
    _outputs_filenames = {}
    _redirect_x = False


class BRAINSTalairachInputSpec(CommandLineInputSpec):
    AC = InputMultiPath(
        traits.Float, desc="Location of AC Point ", sep=",", argstr="--AC %s"
    )
    ACisIndex = traits.Bool(desc="AC Point is Index", argstr="--ACisIndex ")
    PC = InputMultiPath(
        traits.Float, desc="Location of PC Point ", sep=",", argstr="--PC %s"
    )
    PCisIndex = traits.Bool(desc="PC Point is Index", argstr="--PCisIndex ")
    SLA = InputMultiPath(
        traits.Float, desc="Location of SLA Point ", sep=",", argstr="--SLA %s"
    )
    SLAisIndex = traits.Bool(desc="SLA Point is Index", argstr="--SLAisIndex ")
    IRP = InputMultiPath(
        traits.Float, desc="Location of IRP Point ", sep=",", argstr="--IRP %s"
    )
    IRPisIndex = traits.Bool(desc="IRP Point is Index", argstr="--IRPisIndex ")
    inputVolume = File(
        desc="Input image used to define physical space of images",
        exists=True,
        argstr="--inputVolume %s",
    )
    outputBox = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Name of the resulting Talairach Bounding Box file",
        argstr="--outputBox %s",
    )
    outputGrid = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Name of the resulting Talairach Grid file",
        argstr="--outputGrid %s",
    )


class BRAINSTalairachOutputSpec(TraitedSpec):
    outputBox = File(
        desc="Name of the resulting Talairach Bounding Box file", exists=True
    )
    outputGrid = File(desc="Name of the resulting Talairach Grid file", exists=True)


class BRAINSTalairach(SEMLikeCommandLine):
    """title: BRAINS Talairach

    category: BRAINS.Segmentation

    description: This program creates a VTK structured grid defining the Talairach coordinate system based on four points: AC, PC, IRP, and SLA. The resulting structred grid can be written as either a classic VTK file or the new VTK XML file format. Two representations of the resulting grid can be written. The first is a bounding box representation that also contains the location of the AC and PC points. The second representation is the full Talairach grid representation that includes the additional rows of boxes added to the inferior allowing full coverage of the cerebellum.

    version: 0.1

    documentation-url: http://www.nitrc.org/plugins/mwiki/index.php/brains:BRAINSTalairach

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Steven Dunn and Vincent Magnotta

    acknowledgements: Funding for this work was provided by NIH/NINDS award NS050568
    """

    input_spec = BRAINSTalairachInputSpec
    output_spec = BRAINSTalairachOutputSpec
    _cmd = " BRAINSTalairach "
    _outputs_filenames = {"outputGrid": "outputGrid", "outputBox": "outputBox"}
    _redirect_x = False


class BRAINSTalairachMaskInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Input image used to define physical space of resulting mask",
        exists=True,
        argstr="--inputVolume %s",
    )
    talairachParameters = File(
        desc="Name of the Talairach parameter file.",
        exists=True,
        argstr="--talairachParameters %s",
    )
    talairachBox = File(
        desc="Name of the Talairach box file.", exists=True, argstr="--talairachBox %s"
    )
    hemisphereMode = traits.Enum(
        "left",
        "right",
        "both",
        desc="Mode for box creation: left, right, both",
        argstr="--hemisphereMode %s",
    )
    expand = traits.Bool(
        desc="Expand exterior box to include surface CSF", argstr="--expand "
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output filename for the resulting binary image",
        argstr="--outputVolume %s",
    )


class BRAINSTalairachMaskOutputSpec(TraitedSpec):
    outputVolume = File(
        desc="Output filename for the resulting binary image", exists=True
    )


class BRAINSTalairachMask(SEMLikeCommandLine):
    """title: Talairach Mask

    category: BRAINS.Segmentation

    description: This program creates a binary image representing the specified Talairach region. The input is an example image to define the physical space for the resulting image, the Talairach grid representation in VTK format, and the file containing the Talairach box definitions to be generated. These can be combined in BRAINS to create a label map using the procedure Brains::WorkupUtils::CreateLabelMapFromBinaryImages.

    version: 0.1

    documentation-url: http://www.nitrc.org/plugins/mwiki/index.php/brains:BRAINSTalairachMask

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Steven Dunn and Vincent Magnotta

    acknowledgements: Funding for this work was provided by NIH/NINDS award NS050568
    """

    input_spec = BRAINSTalairachMaskInputSpec
    output_spec = BRAINSTalairachMaskOutputSpec
    _cmd = " BRAINSTalairachMask "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The modelgen module provides classes for specifying designs for individual
subject analysis of task-based fMRI experiments. In particular it also includes
algorithms for generating regressors for sparse and sparse-clustered acquisition
experiments.
"""
from copy import deepcopy
import csv, math, os

from nibabel import load
import numpy as np

from ..interfaces.base import (
    BaseInterface,
    TraitedSpec,
    InputMultiPath,
    traits,
    File,
    Bunch,
    BaseInterfaceInputSpec,
    isdefined,
)
from ..utils.filemanip import ensure_list
from ..utils.misc import normalize_mc_params
from .. import config, logging

iflogger = logging.getLogger("nipype.interface")


def spm_hrf(RT, P=None, fMRI_T=16):
    """
    python implementation of spm_hrf

    See ``spm_hrf`` for implementation details::

      % RT   - scan repeat time
      % p    - parameters of the response function (two gamma
      % functions)
      % defaults  (seconds)
      % p(0) - delay of response (relative to onset)       6
      % p(1) - delay of undershoot (relative to onset)    16
      % p(2) - dispersion of response                      1
      % p(3) - dispersion of undershoot                    1
      % p(4) - ratio of response to undershoot             6
      % p(5) - onset (seconds)                             0
      % p(6) - length of kernel (seconds)                 32
      %
      % hrf  - hemodynamic response function
      % p    - parameters of the response function

    The following code using ``scipy.stats.distributions.gamma``
    doesn't return the same result as the ``spm_Gpdf`` function::

        hrf = gamma.pdf(u, p[0]/p[2], scale=dt/p[2]) -
              gamma.pdf(u, p[1]/p[3], scale=dt/p[3])/p[4]

    Example
    -------
    >>> print(spm_hrf(2))
    [  0.00000000e+00   8.65660810e-02   3.74888236e-01   3.84923382e-01
       2.16117316e-01   7.68695653e-02   1.62017720e-03  -3.06078117e-02
      -3.73060781e-02  -3.08373716e-02  -2.05161334e-02  -1.16441637e-02
      -5.82063147e-03  -2.61854250e-03  -1.07732374e-03  -4.10443522e-04
      -1.46257507e-04]

    """
    from scipy.special import gammaln

    p = np.array([6, 16, 1, 1, 6, 0, 32], dtype=float)
    if P is not None:
        p[0 : len(P)] = P

    _spm_Gpdf = lambda x, h, l: np.exp(
        h * np.log(l) + (h - 1) * np.log(x) - (l * x) - gammaln(h)
    )
    # modelled hemodynamic response function - {mixture of Gammas}
    dt = RT / float(fMRI_T)
    u = np.arange(0, int(p[6] / dt + 1)) - p[5] / dt
    with np.errstate(divide="ignore"):  # Known division-by-zero
        hrf = (
            _spm_Gpdf(u, p[0] / p[2], dt / p[2])
            - _spm_Gpdf(u, p[1] / p[3], dt / p[3]) / p[4]
        )
    idx = np.arange(0, int((p[6] / RT) + 1)) * fMRI_T
    hrf = hrf[idx]
    hrf = hrf / np.sum(hrf)
    return hrf


def orth(x_in, y_in):
    """Orthogonalize y_in with respect to x_in.

    >>> orth_expected = np.array([1.7142857142857144, 0.42857142857142883, \
                                  -0.85714285714285676])
    >>> err = np.abs(np.array(orth([1, 2, 3],[4, 5, 6]) - orth_expected))
    >>> all(err < np.finfo(float).eps)
    True

    """
    x = np.array(x_in)[:, None]
    y = np.array(y_in)[:, None]
    y = y - np.dot(x, np.dot(np.linalg.inv(np.dot(x.T, x)), np.dot(x.T, y)))
    if np.linalg.norm(y, 1) > np.exp(-32):
        y = y[:, 0].tolist()
    else:
        y = y_in
    return y


def scale_timings(timelist, input_units, output_units, time_repetition):
    """
    Scale timings given input and output units (scans/secs).

    Parameters
    ----------
    timelist: list of times to scale
    input_units: 'secs' or 'scans'
    output_units: Ibid.
    time_repetition: float in seconds

    """
    if input_units == output_units:
        _scalefactor = 1.0

    if (input_units == "scans") and (output_units == "secs"):
        _scalefactor = time_repetition

    if (input_units == "secs") and (output_units == "scans"):
        _scalefactor = 1.0 / time_repetition
    timelist = [np.max([0.0, _scalefactor * t]) for t in timelist]
    return timelist


def bids_gen_info(
    bids_event_files, condition_column="", amplitude_column=None, time_repetition=False
):
    """
    Generate a subject_info structure from a list of BIDS .tsv event files.

    Parameters
    ----------
    bids_event_files : list of str
        Filenames of BIDS .tsv event files containing columns including:
        'onset', 'duration', and 'trial_type' or the `condition_column` value.
    condition_column : str
        Column of files in `bids_event_files` based on the values of which
        events will be sorted into different regressors
    amplitude_column : str
        Column of files in `bids_event_files` based on the values of which
        to apply amplitudes to events. If unspecified, all events will be
        represented with an amplitude of 1.

    Returns
    -------
    subject_info: list of Bunch

    """
    info = []
    for bids_event_file in bids_event_files:
        with open(bids_event_file) as f:
            f_events = csv.DictReader(f, skipinitialspace=True, delimiter="\t")
            events = [{k: v for k, v in row.items()} for row in f_events]
        if not condition_column:
            condition_column = "_trial_type"
            for i in events:
                i.update({condition_column: "ev0"})
        conditions = sorted(set([i[condition_column] for i in events]))
        runinfo = Bunch(conditions=[], onsets=[], durations=[], amplitudes=[])
        for condition in conditions:
            selected_events = [i for i in events if i[condition_column] == condition]
            onsets = [float(i["onset"]) for i in selected_events]
            durations = [float(i["duration"]) for i in selected_events]
            if time_repetition:
                decimals = math.ceil(-math.log10(time_repetition))
                onsets = [np.round(i, decimals) for i in onsets]
                durations = [np.round(i, decimals) for i in durations]
            runinfo.conditions.append(condition)
            runinfo.onsets.append(onsets)
            runinfo.durations.append(durations)
            try:
                amplitudes = [float(i[amplitude_column]) for i in selected_events]
                runinfo.amplitudes.append(amplitudes)
            except KeyError:
                runinfo.amplitudes.append([1] * len(onsets))
        info.append(runinfo)
    return info


def gen_info(run_event_files):
    """Generate subject_info structure from a list of event files."""
    info = []
    for i, event_files in enumerate(run_event_files):
        runinfo = Bunch(conditions=[], onsets=[], durations=[], amplitudes=[])
        for event_file in event_files:
            _, name = os.path.split(event_file)
            if ".run" in name:
                name, _ = name.split(".run%03d" % (i + 1))
            elif ".txt" in name:
                name, _ = name.split(".txt")

            runinfo.conditions.append(name)
            event_info = np.atleast_2d(np.loadtxt(event_file))
            runinfo.onsets.append(event_info[:, 0].tolist())
            if event_info.shape[1] > 1:
                runinfo.durations.append(event_info[:, 1].tolist())
            else:
                runinfo.durations.append([0])

            if event_info.shape[1] > 2:
                runinfo.amplitudes.append(event_info[:, 2].tolist())
            else:
                delattr(runinfo, "amplitudes")
        info.append(runinfo)
    return info


class SpecifyModelInputSpec(BaseInterfaceInputSpec):
    subject_info = InputMultiPath(
        Bunch,
        mandatory=True,
        xor=["subject_info", "event_files", "bids_event_file"],
        desc="Bunch or List(Bunch) subject-specific "
        "condition information. see "
        ":ref:`nipype.algorithms.modelgen.SpecifyModel` or for details",
    )
    event_files = InputMultiPath(
        traits.List(File(exists=True)),
        mandatory=True,
        xor=["subject_info", "event_files", "bids_event_file"],
        desc="List of event description files 1, 2 or 3 "
        "column format corresponding to onsets, "
        "durations and amplitudes",
    )
    bids_event_file = InputMultiPath(
        File(exists=True),
        mandatory=True,
        xor=["subject_info", "event_files", "bids_event_file"],
        desc="TSV event file containing common BIDS fields: `onset`,"
        "`duration`, and categorization and amplitude columns",
    )
    bids_condition_column = traits.Str(
        default_value="trial_type",
        usedefault=True,
        desc="Column of the file passed to ``bids_event_file`` to the "
        "unique values of which events will be assigned"
        "to regressors",
    )
    bids_amplitude_column = traits.Str(
        desc="Column of the file passed to ``bids_event_file`` "
        "according to which to assign amplitudes to events"
    )
    realignment_parameters = InputMultiPath(
        File(exists=True),
        desc="Realignment parameters returned by motion correction algorithm",
        copyfile=False,
    )
    parameter_source = traits.Enum(
        "SPM",
        "FSL",
        "AFNI",
        "FSFAST",
        "NIPY",
        usedefault=True,
        desc="Source of motion parameters",
    )
    outlier_files = InputMultiPath(
        File(exists=True),
        desc="Files containing scan outlier indices that should be tossed",
        copyfile=False,
    )
    functional_runs = InputMultiPath(
        traits.Either(traits.List(File(exists=True)), File(exists=True)),
        mandatory=True,
        desc="Data files for model. List of 4D "
        "files or list of list of 3D "
        "files per session",
        copyfile=False,
    )
    input_units = traits.Enum(
        "secs",
        "scans",
        mandatory=True,
        desc="Units of event onsets and durations (secs "
        "or scans). Output units are always in secs",
    )
    high_pass_filter_cutoff = traits.Float(
        mandatory=True, desc="High-pass filter cutoff in secs"
    )
    time_repetition = traits.Float(
        mandatory=True,
        desc="Time between the start of one volume "
        "to the start of  the next image volume.",
    )
    # Not implemented yet
    # polynomial_order = traits.Range(0, low=0,
    #        desc ='Number of polynomial functions to model high pass filter.')


class SpecifyModelOutputSpec(TraitedSpec):
    session_info = traits.Any(desc="Session info for level1designs")


class SpecifyModel(BaseInterface):
    """
    Makes a model specification compatible with spm/fsl designers.

    The subject_info field should contain paradigm information in the form of
    a Bunch or a list of Bunch. The Bunch should contain the following
    information::

        [Mandatory]
        conditions : list of names
        onsets : lists of onsets corresponding to each condition
        durations : lists of durations corresponding to each condition. Should be
            left to a single 0 if all events are being modelled as impulses.

        [Optional]
        regressor_names : list of str
            list of names corresponding to each column. Should be None if
            automatically assigned.
        regressors : list of lists
            values for each regressor - must correspond to the number of
            volumes in the functional run
        amplitudes : lists of amplitudes for each event. This will be ignored by
            SPM's Level1Design.

        The following two (tmod, pmod) will be ignored by any Level1Design class
        other than SPM:

        tmod : lists of conditions that should be temporally modulated. Should
            default to None if not being used.
        pmod : list of Bunch corresponding to conditions
          - name : name of parametric modulator
          - param : values of the modulator
          - poly : degree of modulation

    Alternatively, you can provide information through event files.

    The event files have to be in 1, 2 or 3 column format with the columns
    corresponding to Onsets, Durations and Amplitudes and they have to have the
    name event_name.runXXX... e.g.: Words.run001.txt. The event_name part will
    be used to create the condition names.

    Examples
    --------
    >>> from nipype.algorithms import modelgen
    >>> from nipype.interfaces.base import Bunch
    >>> s = modelgen.SpecifyModel()
    >>> s.inputs.input_units = 'secs'
    >>> s.inputs.functional_runs = ['functional2.nii', 'functional3.nii']
    >>> s.inputs.time_repetition = 6
    >>> s.inputs.high_pass_filter_cutoff = 128.
    >>> evs_run2 = Bunch(conditions=['cond1'], onsets=[[2, 50, 100, 180]], durations=[[1]])
    >>> evs_run3 = Bunch(conditions=['cond1'], onsets=[[30, 40, 100, 150]], durations=[[1]])
    >>> s.inputs.subject_info = [evs_run2, evs_run3]

    >>> # Using pmod
    >>> evs_run2 = Bunch(conditions=['cond1', 'cond2'], onsets=[[2, 50], [100, 180]], \
durations=[[0], [0]], pmod=[Bunch(name=['amp'], poly=[2], param=[[1, 2]]), \
None])
    >>> evs_run3 = Bunch(conditions=['cond1', 'cond2'], onsets=[[20, 120], [80, 160]], \
durations=[[0], [0]], pmod=[Bunch(name=['amp'], poly=[2], param=[[1, 2]]), \
None])
    >>> s.inputs.subject_info = [evs_run2, evs_run3]

    """

    input_spec = SpecifyModelInputSpec
    output_spec = SpecifyModelOutputSpec

    def _generate_standard_design(
        self, infolist, functional_runs=None, realignment_parameters=None, outliers=None
    ):
        """Generate a standard design matrix paradigm given information about each run."""
        sessinfo = []
        output_units = "secs"
        if "output_units" in self.inputs.traits():
            output_units = self.inputs.output_units

        for i, info in enumerate(infolist):
            sessinfo.insert(i, dict(cond=[]))
            if isdefined(self.inputs.high_pass_filter_cutoff):
                sessinfo[i]["hpf"] = float(self.inputs.high_pass_filter_cutoff)

            if hasattr(info, "conditions") and info.conditions is not None:
                for cid, cond in enumerate(info.conditions):
                    sessinfo[i]["cond"].insert(cid, dict())
                    sessinfo[i]["cond"][cid]["name"] = info.conditions[cid]
                    scaled_onset = scale_timings(
                        info.onsets[cid],
                        self.inputs.input_units,
                        output_units,
                        self.inputs.time_repetition,
                    )
                    sessinfo[i]["cond"][cid]["onset"] = scaled_onset
                    scaled_duration = scale_timings(
                        info.durations[cid],
                        self.inputs.input_units,
                        output_units,
                        self.inputs.time_repetition,
                    )
                    sessinfo[i]["cond"][cid]["duration"] = scaled_duration
                    if hasattr(info, "amplitudes") and info.amplitudes:
                        sessinfo[i]["cond"][cid]["amplitudes"] = info.amplitudes[cid]

                    if hasattr(info, "tmod") and info.tmod and len(info.tmod) > cid:
                        sessinfo[i]["cond"][cid]["tmod"] = info.tmod[cid]

                    if hasattr(info, "pmod") and info.pmod and len(info.pmod) > cid:
                        if info.pmod[cid]:
                            sessinfo[i]["cond"][cid]["pmod"] = []
                            for j, name in enumerate(info.pmod[cid].name):
                                sessinfo[i]["cond"][cid]["pmod"].insert(j, {})
                                sessinfo[i]["cond"][cid]["pmod"][j]["name"] = name
                                sessinfo[i]["cond"][cid]["pmod"][j]["poly"] = info.pmod[
                                    cid
                                ].poly[j]
                                sessinfo[i]["cond"][cid]["pmod"][j][
                                    "param"
                                ] = info.pmod[cid].param[j]

            sessinfo[i]["regress"] = []
            if hasattr(info, "regressors") and info.regressors is not None:
                for j, r in enumerate(info.regressors):
                    sessinfo[i]["regress"].insert(j, dict(name="", val=[]))
                    if (
                        hasattr(info, "regressor_names")
                        and info.regressor_names is not None
                    ):
                        sessinfo[i]["regress"][j]["name"] = info.regressor_names[j]
                    else:
                        sessinfo[i]["regress"][j]["name"] = "UR%d" % (j + 1)
                    sessinfo[i]["regress"][j]["val"] = info.regressors[j]
            sessinfo[i]["scans"] = functional_runs[i]

        if realignment_parameters is not None:
            for i, rp in enumerate(realignment_parameters):
                mc = realignment_parameters[i]
                for col in range(mc.shape[1]):
                    colidx = len(sessinfo[i]["regress"])
                    sessinfo[i]["regress"].insert(colidx, dict(name="", val=[]))
                    sessinfo[i]["regress"][colidx]["name"] = "Realign%d" % (col + 1)
                    sessinfo[i]["regress"][colidx]["val"] = mc[:, col].tolist()

        if outliers is not None:
            for i, out in enumerate(outliers):
                numscans = 0
                for f in ensure_list(sessinfo[i]["scans"]):
                    shape = load(f).shape
                    if len(shape) == 3 or shape[3] == 1:
                        iflogger.warning(
                            "You are using 3D instead of 4D "
                            "files. Are you sure this was "
                            "intended?"
                        )
                        numscans += 1
                    else:
                        numscans += shape[3]

                for j, scanno in enumerate(out):
                    colidx = len(sessinfo[i]["regress"])
                    sessinfo[i]["regress"].insert(colidx, dict(name="", val=[]))
                    sessinfo[i]["regress"][colidx]["name"] = "Outlier%d" % (j + 1)
                    sessinfo[i]["regress"][colidx]["val"] = np.zeros((1, numscans))[
                        0
                    ].tolist()
                    sessinfo[i]["regress"][colidx]["val"][int(scanno)] = 1
        return sessinfo

    def _generate_design(self, infolist=None):
        """Generate design specification for a typical fmri paradigm"""
        realignment_parameters = []
        if isdefined(self.inputs.realignment_parameters):
            for parfile in self.inputs.realignment_parameters:
                realignment_parameters.append(
                    np.apply_along_axis(
                        func1d=normalize_mc_params,
                        axis=1,
                        arr=np.loadtxt(parfile),
                        source=self.inputs.parameter_source,
                    )
                )
        outliers = []
        if isdefined(self.inputs.outlier_files):
            for filename in self.inputs.outlier_files:
                try:
                    outindices = np.loadtxt(filename, dtype=int)
                except IOError:
                    outliers.append([])
                else:
                    if outindices.size == 1:
                        outliers.append([outindices.tolist()])
                    else:
                        outliers.append(outindices.tolist())

        if infolist is None:
            if isdefined(self.inputs.subject_info):
                infolist = self.inputs.subject_info
            elif isdefined(self.inputs.event_files):
                infolist = gen_info(self.inputs.event_files)
            elif isdefined(self.inputs.bids_event_file):
                infolist = bids_gen_info(
                    self.inputs.bids_event_file,
                    self.inputs.bids_condition_column,
                    self.inputs.bids_amplitude_column,
                    self.inputs.time_repetition,
                )
        self._sessinfo = self._generate_standard_design(
            infolist,
            functional_runs=self.inputs.functional_runs,
            realignment_parameters=realignment_parameters,
            outliers=outliers,
        )

    def _run_interface(self, runtime):
        """ """
        self._sessioninfo = None
        self._generate_design()
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        if not hasattr(self, "_sessinfo"):
            self._generate_design()
        outputs["session_info"] = self._sessinfo

        return outputs


class SpecifySPMModelInputSpec(SpecifyModelInputSpec):
    concatenate_runs = traits.Bool(
        False,
        usedefault=True,
        desc="Concatenate all runs to look like a single session.",
    )
    output_units = traits.Enum(
        "secs",
        "scans",
        usedefault=True,
        desc="Units of design event onsets and durations (secs or scans)",
    )


class SpecifySPMModel(SpecifyModel):
    """Add SPM specific options to SpecifyModel

    Adds:

       - concatenate_runs
       - output_units

    Examples
    --------
    >>> from nipype.algorithms import modelgen
    >>> from nipype.interfaces.base import Bunch
    >>> s = modelgen.SpecifySPMModel()
    >>> s.inputs.input_units = 'secs'
    >>> s.inputs.output_units = 'scans'
    >>> s.inputs.high_pass_filter_cutoff = 128.
    >>> s.inputs.functional_runs = ['functional2.nii', 'functional3.nii']
    >>> s.inputs.time_repetition = 6
    >>> s.inputs.concatenate_runs = True
    >>> evs_run2 = Bunch(conditions=['cond1'], onsets=[[2, 50, 100, 180]], durations=[[1]])
    >>> evs_run3 = Bunch(conditions=['cond1'], onsets=[[30, 40, 100, 150]], durations=[[1]])
    >>> s.inputs.subject_info = [evs_run2, evs_run3]

    """

    input_spec = SpecifySPMModelInputSpec

    def _concatenate_info(self, infolist):
        nscans = []
        for i, f in enumerate(self.inputs.functional_runs):
            if isinstance(f, list):
                numscans = len(f)
            elif isinstance(f, (str, bytes)):
                img = load(f)
                numscans = img.shape[3]
            else:
                raise Exception("Functional input not specified correctly")
            nscans.insert(i, numscans)

        # now combine all fields into 1
        # names, onsets, durations, amplitudes, pmod, tmod, regressor_names,
        # regressors
        infoout = infolist[0]
        for j, val in enumerate(infolist[0].durations):
            if len(infolist[0].onsets[j]) > 1 and len(val) == 1:
                infoout.durations[j] = infolist[0].durations[j] * len(
                    infolist[0].onsets[j]
                )

        for i, info in enumerate(infolist[1:]):
            # info.[conditions, tmod] remain the same
            if info.onsets:
                for j, val in enumerate(info.onsets):
                    if self.inputs.input_units == "secs":
                        onsets = np.array(
                            info.onsets[j]
                        ) + self.inputs.time_repetition * sum(nscans[0 : (i + 1)])
                        infoout.onsets[j].extend(onsets.tolist())
                    else:
                        onsets = np.array(info.onsets[j]) + sum(nscans[0 : (i + 1)])
                        infoout.onsets[j].extend(onsets.tolist())

                for j, val in enumerate(info.durations):
                    if len(info.onsets[j]) > 1 and len(val) == 1:
                        infoout.durations[j].extend(
                            info.durations[j] * len(info.onsets[j])
                        )
                    elif len(info.onsets[j]) == len(val):
                        infoout.durations[j].extend(info.durations[j])
                    else:
                        raise ValueError(
                            "Mismatch in number of onsets and \
                                          durations for run {0}, condition \
                                          {1}".format(
                                i + 2, j + 1
                            )
                        )

                if hasattr(info, "amplitudes") and info.amplitudes:
                    for j, val in enumerate(info.amplitudes):
                        infoout.amplitudes[j].extend(info.amplitudes[j])

                if hasattr(info, "pmod") and info.pmod:
                    for j, val in enumerate(info.pmod):
                        if val:
                            for key, data in enumerate(val.param):
                                infoout.pmod[j].param[key].extend(data)

            if hasattr(info, "regressors") and info.regressors:
                # assumes same ordering of regressors across different
                # runs and the same names for the regressors
                for j, v in enumerate(info.regressors):
                    infoout.regressors[j].extend(info.regressors[j])

            # insert session regressors
            if not hasattr(infoout, "regressors") or not infoout.regressors:
                infoout.regressors = []
            onelist = np.zeros((1, sum(nscans)))
            onelist[0, sum(nscans[0:i]) : sum(nscans[0 : (i + 1)])] = 1
            infoout.regressors.insert(len(infoout.regressors), onelist.tolist()[0])
        return [infoout], nscans

    def _generate_design(self, infolist=None):
        if (
            not isdefined(self.inputs.concatenate_runs)
            or not self.inputs.concatenate_runs
        ):
            super(SpecifySPMModel, self)._generate_design(infolist=infolist)
            return

        if isdefined(self.inputs.subject_info):
            infolist = self.inputs.subject_info
        else:
            infolist = gen_info(self.inputs.event_files)
        concatlist, nscans = self._concatenate_info(infolist)
        functional_runs = [ensure_list(self.inputs.functional_runs)]
        realignment_parameters = []
        if isdefined(self.inputs.realignment_parameters):
            realignment_parameters = []
            for parfile in self.inputs.realignment_parameters:
                mc = np.apply_along_axis(
                    func1d=normalize_mc_params,
                    axis=1,
                    arr=np.loadtxt(parfile),
                    source=self.inputs.parameter_source,
                )
                if not realignment_parameters:
                    realignment_parameters.insert(0, mc)
                else:
                    realignment_parameters[0] = np.concatenate(
                        (realignment_parameters[0], mc)
                    )
        outliers = []
        if isdefined(self.inputs.outlier_files):
            outliers = [[]]
            for i, filename in enumerate(self.inputs.outlier_files):
                try:
                    out = np.loadtxt(filename)
                except IOError:
                    iflogger.warning("Error reading outliers file %s", filename)
                    out = np.array([])

                if out.size > 0:
                    iflogger.debug(
                        "fname=%s, out=%s, nscans=%d", filename, out, sum(nscans[0:i])
                    )
                    sumscans = out.astype(int) + sum(nscans[0:i])

                    if out.size == 1:
                        outliers[0] += [np.array(sumscans, dtype=int).tolist()]
                    else:
                        outliers[0] += np.array(sumscans, dtype=int).tolist()

        self._sessinfo = self._generate_standard_design(
            concatlist,
            functional_runs=functional_runs,
            realignment_parameters=realignment_parameters,
            outliers=outliers,
        )


class SpecifySparseModelInputSpec(SpecifyModelInputSpec):
    time_acquisition = traits.Float(
        0, mandatory=True, desc="Time in seconds to acquire a single image volume"
    )
    volumes_in_cluster = traits.Range(
        1, usedefault=True, desc="Number of scan volumes in a cluster"
    )
    model_hrf = traits.Bool(desc="Model sparse events with hrf")
    stimuli_as_impulses = traits.Bool(
        True, desc="Treat each stimulus to be impulse-like", usedefault=True
    )
    use_temporal_deriv = traits.Bool(
        requires=["model_hrf"],
        desc="Create a temporal derivative in addition to regular regressor",
    )
    scale_regressors = traits.Bool(
        True, desc="Scale regressors by the peak", usedefault=True
    )
    scan_onset = traits.Float(
        0.0, desc="Start of scanning relative to onset of run in secs", usedefault=True
    )
    save_plot = traits.Bool(
        desc=("Save plot of sparse design calculation (requires matplotlib)")
    )


class SpecifySparseModelOutputSpec(SpecifyModelOutputSpec):
    sparse_png_file = File(desc="PNG file showing sparse design")
    sparse_svg_file = File(desc="SVG file showing sparse design")


class SpecifySparseModel(SpecifyModel):
    """Specify a sparse model that is compatible with SPM/FSL designers [1]_.

    Examples
    --------
    >>> from nipype.algorithms import modelgen
    >>> from nipype.interfaces.base import Bunch
    >>> s = modelgen.SpecifySparseModel()
    >>> s.inputs.input_units = 'secs'
    >>> s.inputs.functional_runs = ['functional2.nii', 'functional3.nii']
    >>> s.inputs.time_repetition = 6
    >>> s.inputs.time_acquisition = 2
    >>> s.inputs.high_pass_filter_cutoff = 128.
    >>> s.inputs.model_hrf = True
    >>> evs_run2 = Bunch(conditions=['cond1'], onsets=[[2, 50, 100, 180]],
    ...                  durations=[[1]])
    >>> evs_run3 = Bunch(conditions=['cond1'], onsets=[[30, 40, 100, 150]],
    ...                  durations=[[1]])
    >>> s.inputs.subject_info = [evs_run2, evs_run3]  # doctest: +SKIP

    References
    ----------
    .. [1] Perrachione TK and Ghosh SS (2013) Optimized design and analysis of
       sparse-sampling fMRI experiments. Front. Neurosci. 7:55
       http://journal.frontiersin.org/Journal/10.3389/fnins.2013.00055/abstract

    """

    input_spec = SpecifySparseModelInputSpec
    output_spec = SpecifySparseModelOutputSpec

    def _gen_regress(self, i_onsets, i_durations, i_amplitudes, nscans):
        """Generates a regressor for a sparse/clustered-sparse acquisition"""
        bplot = False
        if isdefined(self.inputs.save_plot) and self.inputs.save_plot:
            bplot = True
            import matplotlib

            matplotlib.use(config.get("execution", "matplotlib_backend"))
            import matplotlib.pyplot as plt

        TR = int(np.round(self.inputs.time_repetition * 1000))  # in ms
        if self.inputs.time_acquisition:
            TA = int(np.round(self.inputs.time_acquisition * 1000))  # in ms
        else:
            TA = TR  # in ms
        nvol = self.inputs.volumes_in_cluster
        SCANONSET = np.round(self.inputs.scan_onset * 1000)
        total_time = TR * (nscans - nvol) / nvol + TA * nvol + SCANONSET
        SILENCE = TR - TA * nvol
        dt = TA / 10.0
        durations = np.round(np.array(i_durations) * 1000)
        if len(durations) == 1:
            durations = durations * np.ones((len(i_onsets)))
        onsets = np.round(np.array(i_onsets) * 1000)
        dttemp = math.gcd(TA, math.gcd(SILENCE, TR))
        if dt < dttemp:
            if dttemp % dt != 0:
                dt = float(math.gcd(dttemp, int(dt)))

        if dt < 1:
            raise Exception("Time multiple less than 1 ms")
        iflogger.info("Setting dt = %d ms\n", dt)
        npts = int(np.ceil(total_time / dt))
        times = np.arange(0, total_time, dt) * 1e-3
        timeline = np.zeros((npts))
        timeline2 = np.zeros((npts))
        if isdefined(self.inputs.model_hrf) and self.inputs.model_hrf:
            hrf = spm_hrf(dt * 1e-3)
        reg_scale = 1.0
        if self.inputs.scale_regressors:
            boxcar = np.zeros(int(50.0 * 1e3 / dt))
            if self.inputs.stimuli_as_impulses:
                boxcar[int(1.0 * 1e3 / dt)] = 1.0
                reg_scale = float(TA / dt)
            else:
                boxcar[int(1.0 * 1e3 / dt) : int(2.0 * 1e3 / dt)] = 1.0

            if isdefined(self.inputs.model_hrf) and self.inputs.model_hrf:
                response = np.convolve(boxcar, hrf)
                reg_scale = 1.0 / response.max()
                iflogger.info(
                    "response sum: %.4f max: %.4f", response.sum(), response.max()
                )
            iflogger.info("reg_scale: %.4f", reg_scale)

        for i, t in enumerate(onsets):
            idx = int(np.round(t / dt))
            if i_amplitudes:
                if len(i_amplitudes) > 1:
                    timeline2[idx] = i_amplitudes[i]
                else:
                    timeline2[idx] = i_amplitudes[0]
            else:
                timeline2[idx] = 1

            if bplot:
                plt.subplot(4, 1, 1)
                plt.plot(times, timeline2)

            if not self.inputs.stimuli_as_impulses:
                if durations[i] == 0:
                    durations[i] = TA * nvol
                stimdur = np.ones((int(durations[i] / dt)))
                timeline2 = np.convolve(timeline2, stimdur)[0 : len(timeline2)]
            timeline += timeline2
            timeline2[:] = 0

        if bplot:
            plt.subplot(4, 1, 2)
            plt.plot(times, timeline)

        if isdefined(self.inputs.model_hrf) and self.inputs.model_hrf:
            timeline = np.convolve(timeline, hrf)[0 : len(timeline)]
            if (
                isdefined(self.inputs.use_temporal_deriv)
                and self.inputs.use_temporal_deriv
            ):
                # create temporal deriv
                timederiv = np.concatenate(([0], np.diff(timeline)))

        if bplot:
            plt.subplot(4, 1, 3)
            plt.plot(times, timeline)
            if (
                isdefined(self.inputs.use_temporal_deriv)
                and self.inputs.use_temporal_deriv
            ):
                plt.plot(times, timederiv)
        # sample timeline
        timeline2 = np.zeros((npts))
        reg = []
        regderiv = []
        for i, trial in enumerate(np.arange(nscans) / nvol):
            scanstart = int((SCANONSET + trial * TR + (i % nvol) * TA) / dt)
            scanidx = scanstart + np.arange(int(TA / dt))
            timeline2[scanidx] = np.max(timeline)
            reg.insert(i, np.mean(timeline[scanidx]) * reg_scale)
            if (
                isdefined(self.inputs.use_temporal_deriv)
                and self.inputs.use_temporal_deriv
            ):
                regderiv.insert(i, np.mean(timederiv[scanidx]) * reg_scale)

        if isdefined(self.inputs.use_temporal_deriv) and self.inputs.use_temporal_deriv:
            iflogger.info("orthoganlizing derivative w.r.t. main regressor")
            regderiv = orth(reg, regderiv)

        if bplot:
            plt.subplot(4, 1, 3)
            plt.plot(times, timeline2)
            plt.subplot(4, 1, 4)
            plt.bar(np.arange(len(reg)), reg, width=0.5)
            plt.savefig("sparse.png")
            plt.savefig("sparse.svg")

        if regderiv:
            return [reg, regderiv]
        else:
            return reg

    def _cond_to_regress(self, info, nscans):
        """Converts condition information to full regressors"""
        reg = []
        regnames = []
        for i, cond in enumerate(info.conditions):
            if hasattr(info, "amplitudes") and info.amplitudes:
                amplitudes = info.amplitudes[i]
            else:
                amplitudes = None
            regnames.insert(len(regnames), cond)
            scaled_onsets = scale_timings(
                info.onsets[i],
                self.inputs.input_units,
                "secs",
                self.inputs.time_repetition,
            )
            scaled_durations = scale_timings(
                info.durations[i],
                self.inputs.input_units,
                "secs",
                self.inputs.time_repetition,
            )
            regressor = self._gen_regress(
                scaled_onsets, scaled_durations, amplitudes, nscans
            )
            if (
                isdefined(self.inputs.use_temporal_deriv)
                and self.inputs.use_temporal_deriv
            ):
                reg.insert(len(reg), regressor[0])
                regnames.insert(len(regnames), cond + "_D")
                reg.insert(len(reg), regressor[1])
            else:
                reg.insert(len(reg), regressor)
        # need to deal with temporal and parametric modulators
        # for sparse-clustered acquisitions enter T1-effect regressors
        nvol = self.inputs.volumes_in_cluster
        if nvol > 1:
            for i in range(nvol - 1):
                treg = np.zeros((nscans / nvol, nvol))
                treg[:, i] = 1
                reg.insert(len(reg), treg.ravel().tolist())
                regnames.insert(len(regnames), "T1effect_%d" % i)
        return reg, regnames

    def _generate_clustered_design(self, infolist):
        """Generates condition information for sparse-clustered
        designs.

        """
        infoout = deepcopy(infolist)
        for i, info in enumerate(infolist):
            infoout[i].conditions = None
            infoout[i].onsets = None
            infoout[i].durations = None
            if info.conditions:
                img = load(self.inputs.functional_runs[i])
                nscans = img.shape[3]
                reg, regnames = self._cond_to_regress(info, nscans)
                if hasattr(infoout[i], "regressors") and infoout[i].regressors:
                    if not infoout[i].regressor_names:
                        infoout[i].regressor_names = [
                            "R%d" % j for j in range(len(infoout[i].regressors))
                        ]
                else:
                    infoout[i].regressors = []
                    infoout[i].regressor_names = []

                for j, r in enumerate(reg):
                    regidx = len(infoout[i].regressors)
                    infoout[i].regressor_names.insert(regidx, regnames[j])
                    infoout[i].regressors.insert(regidx, r)
        return infoout

    def _generate_design(self, infolist=None):
        if isdefined(self.inputs.subject_info):
            infolist = self.inputs.subject_info
        else:
            infolist = gen_info(self.inputs.event_files)
        sparselist = self._generate_clustered_design(infolist)
        super(SpecifySparseModel, self)._generate_design(infolist=sparselist)

    def _list_outputs(self):
        outputs = self._outputs().get()
        if not hasattr(self, "_sessinfo"):
            self._generate_design()
        outputs["session_info"] = self._sessinfo

        if isdefined(self.inputs.save_plot) and self.inputs.save_plot:
            outputs["sparse_png_file"] = os.path.join(os.getcwd(), "sparse.png")
            outputs["sparse_svg_file"] = os.path.join(os.getcwd(), "sparse.svg")
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import ImageMeants


def test_ImageMeants_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        eig=dict(
            argstr="--eig",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
            position=0,
        ),
        mask=dict(
            argstr="-m %s",
            extensions=None,
        ),
        nobin=dict(
            argstr="--no_bin",
        ),
        order=dict(
            argstr="--order=%d",
            usedefault=True,
        ),
        out_file=dict(
            argstr="-o %s",
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_type=dict(),
        show_all=dict(
            argstr="--showall",
        ),
        spatial_coord=dict(
            argstr="-c %s",
        ),
        transpose=dict(
            argstr="--transpose",
        ),
        use_mm=dict(
            argstr="--usemm",
        ),
    )
    inputs = ImageMeants.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ImageMeants_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = ImageMeants.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import InvWarp


def test_InvWarp_inputs():
    input_map = dict(
        absolute=dict(
            argstr="--abs",
            xor=["relative"],
        ),
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inverse_warp=dict(
            argstr="--out=%s",
            extensions=None,
            hash_files=False,
            name_source=["warp"],
            name_template="%s_inverse",
        ),
        jacobian_max=dict(
            argstr="--jmax=%f",
        ),
        jacobian_min=dict(
            argstr="--jmin=%f",
        ),
        niter=dict(
            argstr="--niter=%d",
        ),
        noconstraint=dict(
            argstr="--noconstraint",
        ),
        output_type=dict(),
        reference=dict(
            argstr="--ref=%s",
            extensions=None,
            mandatory=True,
        ),
        regularise=dict(
            argstr="--regularise=%f",
        ),
        relative=dict(
            argstr="--rel",
            xor=["absolute"],
        ),
        warp=dict(
            argstr="--warp=%s",
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = InvWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_InvWarp_outputs():
    output_map = dict(
        inverse_warp=dict(
            extensions=None,
        ),
    )
    outputs = InvWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import pytest

from ....testing import example_data
from ...niftyreg import get_custom_path
from ...niftyreg.tests.test_regutils import no_nifty_tool
from .. import FillLesions


@pytest.mark.skipif(
    no_nifty_tool(cmd="seg_FillLesions"), reason="niftyseg is not installed"
)
def test_seg_filllesions():

    # Create a node object
    seg_fill = FillLesions()

    # Check if the command is properly defined
    cmd = get_custom_path("seg_FillLesions", env_dir="NIFTYSEGDIR")
    assert seg_fill.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        seg_fill.run()

    # Assign some input data
    in_file = example_data("im1.nii")
    lesion_mask = example_data("im2.nii")
    seg_fill.inputs.in_file = in_file
    seg_fill.inputs.lesion_mask = lesion_mask

    expected_cmd = "{cmd} -i {in_file} -l {lesion_mask} -o {out_file}".format(
        cmd=cmd,
        in_file=in_file,
        lesion_mask=lesion_mask,
        out_file="im1_lesions_filled.nii.gz",
    )

    assert seg_fill.cmdline == expected_cmd



# -*- coding: utf-8 -*-
import os
import nipype.interfaces.base as nib

import pytest
import nipype.pipeline.engine as pe


class InputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(desc="a random int")
    input2 = nib.traits.Int(desc="a random int")


class OutputSpec(nib.TraitedSpec):
    output1 = nib.traits.List(nib.traits.Int, desc="outputs")


class DebugTestInterface(nib.BaseInterface):
    input_spec = InputSpec
    output_spec = OutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["output1"] = [1, self.inputs.input1]
        return outputs


def callme(node, graph):
    pass


def test_debug(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name="pipe")
    mod1 = pe.Node(DebugTestInterface(), name="mod1")
    mod2 = pe.MapNode(DebugTestInterface(), iterfield=["input1"], name="mod2")

    pipe.connect([(mod1, mod2, [("output1", "input1")])])
    pipe.base_dir = os.getcwd()
    mod1.inputs.input1 = 1

    run_wf = lambda: pipe.run(plugin="Debug")
    with pytest.raises(ValueError):
        run_wf()

    exc = None
    try:
        pipe.run(plugin="Debug", plugin_args={"callable": callme})
    except Exception as e:
        exc = e

    assert exc is None, "unexpected exception caught"



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..convert import CFFConverter


def test_CFFConverter_inputs():
    input_map = dict(
        creator=dict(),
        data_files=dict(),
        description=dict(
            usedefault=True,
        ),
        email=dict(),
        gifti_labels=dict(),
        gifti_surfaces=dict(),
        gpickled_networks=dict(),
        graphml_networks=dict(),
        license=dict(),
        nifti_volumes=dict(),
        out_file=dict(
            extensions=None,
            usedefault=True,
        ),
        publisher=dict(),
        references=dict(),
        relation=dict(),
        rights=dict(),
        script_files=dict(),
        species=dict(
            usedefault=True,
        ),
        timeseries_files=dict(),
        title=dict(),
        tract_files=dict(),
    )
    inputs = CFFConverter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_CFFConverter_outputs():
    output_map = dict(
        connectome_file=dict(
            extensions=None,
        ),
    )
    outputs = CFFConverter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..specialized import RobustStatisticsSegmenter


def test_RobustStatisticsSegmenter_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        curvatureWeight=dict(
            argstr="--curvatureWeight %f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expectedVolume=dict(
            argstr="--expectedVolume %f",
        ),
        intensityHomogeneity=dict(
            argstr="--intensityHomogeneity %f",
        ),
        labelImageFileName=dict(
            argstr="%s",
            extensions=None,
            position=-2,
        ),
        labelValue=dict(
            argstr="--labelValue %d",
        ),
        maxRunningTime=dict(
            argstr="--maxRunningTime %f",
        ),
        originalImageFileName=dict(
            argstr="%s",
            extensions=None,
            position=-3,
        ),
        segmentedImageFileName=dict(
            argstr="%s",
            hash_files=False,
            position=-1,
        ),
    )
    inputs = RobustStatisticsSegmenter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_RobustStatisticsSegmenter_outputs():
    output_map = dict(
        segmentedImageFileName=dict(
            extensions=None,
            position=-1,
        ),
    )
    outputs = RobustStatisticsSegmenter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..developer import JistBrainMgdmSegmentation


def test_JistBrainMgdmSegmentation_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inAdjust=dict(
            argstr="--inAdjust %s",
        ),
        inAtlas=dict(
            argstr="--inAtlas %s",
            extensions=None,
        ),
        inCompute=dict(
            argstr="--inCompute %s",
        ),
        inCurvature=dict(
            argstr="--inCurvature %f",
        ),
        inData=dict(
            argstr="--inData %f",
        ),
        inFLAIR=dict(
            argstr="--inFLAIR %s",
            extensions=None,
        ),
        inMP2RAGE=dict(
            argstr="--inMP2RAGE %s",
            extensions=None,
        ),
        inMP2RAGE2=dict(
            argstr="--inMP2RAGE2 %s",
            extensions=None,
        ),
        inMPRAGE=dict(
            argstr="--inMPRAGE %s",
            extensions=None,
        ),
        inMax=dict(
            argstr="--inMax %d",
        ),
        inMin=dict(
            argstr="--inMin %f",
        ),
        inOutput=dict(
            argstr="--inOutput %s",
        ),
        inPV=dict(
            argstr="--inPV %s",
            extensions=None,
        ),
        inPosterior=dict(
            argstr="--inPosterior %f",
        ),
        inSteps=dict(
            argstr="--inSteps %d",
        ),
        inTopology=dict(
            argstr="--inTopology %s",
        ),
        null=dict(
            argstr="--null %s",
        ),
        outLevelset=dict(
            argstr="--outLevelset %s",
            hash_files=False,
        ),
        outPosterior2=dict(
            argstr="--outPosterior2 %s",
            hash_files=False,
        ),
        outPosterior3=dict(
            argstr="--outPosterior3 %s",
            hash_files=False,
        ),
        outSegmented=dict(
            argstr="--outSegmented %s",
            hash_files=False,
        ),
        xDefaultMem=dict(
            argstr="-xDefaultMem %d",
        ),
        xMaxProcess=dict(
            argstr="-xMaxProcess %d",
            usedefault=True,
        ),
        xPrefExt=dict(
            argstr="--xPrefExt %s",
        ),
    )
    inputs = JistBrainMgdmSegmentation.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_JistBrainMgdmSegmentation_outputs():
    output_map = dict(
        outLevelset=dict(
            extensions=None,
        ),
        outPosterior2=dict(
            extensions=None,
        ),
        outPosterior3=dict(
            extensions=None,
        ),
        outSegmented=dict(
            extensions=None,
        ),
    )
    outputs = JistBrainMgdmSegmentation.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from ...testing import utils
from ..confounds import TSNR
from .. import misc

import pytest
import numpy.testing as npt
from unittest import mock
import nibabel as nb
import numpy as np
import os


class TestTSNR:
    """Note: Tests currently do a poor job of testing functionality"""

    in_filenames = {"in_file": "tsnrinfile.nii"}

    out_filenames = {  # default output file names
        "detrended_file": "detrend.nii.gz",
        "mean_file": "mean.nii.gz",
        "stddev_file": "stdev.nii.gz",
        "tsnr_file": "tsnr.nii.gz",
    }

    @pytest.fixture(autouse=True)
    def setup_class(self, tmpdir):
        # setup temp folder
        tmpdir.chdir()

        utils.save_toy_nii(self.fake_data, self.in_filenames["in_file"])

    def test_tsnr(self):
        # run
        tsnrresult = TSNR(in_file=self.in_filenames["in_file"]).run()

        # assert
        self.assert_expected_outputs(
            tsnrresult,
            {
                "mean_file": (2.8, 7.4),
                "stddev_file": (0.8, 2.9),
                "tsnr_file": (1.3, 9.25),
            },
        )

    def test_tsnr_withpoly1(self):
        # run
        tsnrresult = TSNR(in_file=self.in_filenames["in_file"], regress_poly=1).run()

        # assert
        self.assert_expected_outputs_poly(
            tsnrresult,
            {
                "detrended_file": (-0.1, 8.7),
                "mean_file": (2.8, 7.4),
                "stddev_file": (0.75, 2.75),
                "tsnr_file": (1.4, 9.9),
            },
        )

    def test_tsnr_withpoly2(self):
        # run
        tsnrresult = TSNR(in_file=self.in_filenames["in_file"], regress_poly=2).run()

        # assert
        self.assert_expected_outputs_poly(
            tsnrresult,
            {
                "detrended_file": (-0.22, 8.55),
                "mean_file": (2.8, 7.7),
                "stddev_file": (0.21, 2.4),
                "tsnr_file": (1.7, 35.9),
            },
        )

    def test_tsnr_withpoly3(self):
        # run
        tsnrresult = TSNR(in_file=self.in_filenames["in_file"], regress_poly=3).run()

        # assert
        self.assert_expected_outputs_poly(
            tsnrresult,
            {
                "detrended_file": (1.8, 7.95),
                "mean_file": (2.8, 7.7),
                "stddev_file": (0.1, 1.7),
                "tsnr_file": (2.6, 57.3),
            },
        )

    @mock.patch("warnings.warn")
    def test_warning(self, mock_warn):
        """test that usage of misc.TSNR trips a warning to use
        confounds.TSNR instead"""
        # run
        misc.TSNR(in_file=self.in_filenames["in_file"])

        # assert
        assert True in [
            args[0].count("confounds") > 0 for _, args, _ in mock_warn.mock_calls
        ]

    def assert_expected_outputs_poly(self, tsnrresult, expected_ranges):
        assert (
            os.path.basename(tsnrresult.outputs.detrended_file)
            == self.out_filenames["detrended_file"]
        )
        self.assert_expected_outputs(tsnrresult, expected_ranges)

    def assert_expected_outputs(self, tsnrresult, expected_ranges):
        self.assert_default_outputs(tsnrresult.outputs)
        self.assert_unchanged(expected_ranges)

    def assert_default_outputs(self, outputs):
        assert os.path.basename(outputs.mean_file) == self.out_filenames["mean_file"]
        assert (
            os.path.basename(outputs.stddev_file) == self.out_filenames["stddev_file"]
        )
        assert os.path.basename(outputs.tsnr_file) == self.out_filenames["tsnr_file"]

    def assert_unchanged(self, expected_ranges):
        for key, (min_, max_) in expected_ranges.items():
            data = np.asarray(nb.load(self.out_filenames[key]).dataobj)
            npt.assert_almost_equal(np.amin(data), min_, decimal=1)
            npt.assert_almost_equal(np.amax(data), max_, decimal=1)

    fake_data = np.array(
        [
            [[[2, 4, 3, 9, 1], [3, 6, 4, 7, 4]], [[8, 3, 4, 6, 2], [4, 0, 4, 4, 2]]],
            [[[9, 7, 5, 5, 7], [7, 8, 4, 8, 4]], [[0, 4, 7, 1, 7], [6, 8, 8, 8, 7]]],
        ]
    )



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""SPM is a software package for the analysis of brain imaging data sequences."""
from .base import Info, SPMCommand, logger, no_spm, scans_for_fname, scans_for_fnames
from .preprocess import (
    ApplyVDM,
    FieldMap,
    SliceTiming,
    Realign,
    RealignUnwarp,
    Coregister,
    Normalize,
    Normalize12,
    Segment,
    Smooth,
    NewSegment,
    MultiChannelNewSegment,
    DARTEL,
    DARTELNorm2MNI,
    CreateWarped,
    VBMSegment,
)
from .model import (
    Level1Design,
    EstimateModel,
    EstimateContrast,
    Threshold,
    OneSampleTTestDesign,
    TwoSampleTTestDesign,
    PairedTTestDesign,
    MultipleRegressionDesign,
)
from .utils import (
    Analyze2nii,
    CalcCoregAffine,
    ApplyTransform,
    Reslice,
    ApplyInverseDeformation,
    ResliceToReference,
    DicomImport,
)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import MaskTool


def test_MaskTool_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        count=dict(
            argstr="-count",
            position=2,
        ),
        datum=dict(
            argstr="-datum %s",
        ),
        dilate_inputs=dict(
            argstr="-dilate_inputs %s",
        ),
        dilate_results=dict(
            argstr="-dilate_results %s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fill_dirs=dict(
            argstr="-fill_dirs %s",
            requires=["fill_holes"],
        ),
        fill_holes=dict(
            argstr="-fill_holes",
        ),
        frac=dict(
            argstr="-frac %s",
        ),
        in_file=dict(
            argstr="-input %s",
            copyfile=False,
            mandatory=True,
            position=-1,
        ),
        inter=dict(
            argstr="-inter",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_mask",
        ),
        outputtype=dict(),
        union=dict(
            argstr="-union",
        ),
        verbose=dict(
            argstr="-verb %s",
        ),
    )
    inputs = MaskTool.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MaskTool_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = MaskTool.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
